import { CircularList } from "../HistoryList.js";
import { Util } from "../Util.js";
import { Vector } from "../Vector.js";
import { assertEquals, schedule, startTest, assertThrows, assertTrue, assertFalse, assertNull, assertNotNull } from "../../../test/TestRig.js";
const groupName = 'CircularListTest.';
export default function scheduleTests() {
    schedule(testUtilCircularList1);
    schedule(testUtilCircularList2);
    schedule(testUtilCircularList3);
}
;
function testUtilCircularList1() {
    startTest(groupName + 'testUtilCircularList1');
    const cList = new CircularList(10);
    let i, j;
    assertEquals(0, cList.getSize());
    assertEquals(0, cList.getStartIndex());
    assertEquals(-1, cList.getEndIndex());
    assertNull(cList.getEndValue());
    let cIter = cList.getIterator();
    assertFalse(cIter.hasNext());
    assertFalse(cIter.hasPrevious());
    assertThrows(() => cIter.nextValue());
    assertThrows(() => cIter.previousValue());
    assertThrows(() => cIter.getIndex());
    j = cList.store(new Vector(0, 1));
    assertEquals(0, j);
    assertEquals(1, cList.getSize());
    assertEquals(0, cList.getStartIndex());
    assertEquals(0, cList.getEndIndex());
    let endValue;
    assertNotNull(endValue = cList.getEndValue());
    if (endValue)
        assertTrue(endValue.equals(new Vector(0, 1)));
    cIter = cList.getIterator();
    assertTrue(cIter.hasNext());
    cIter.nextValue();
    assertEquals(0, cIter.getIndex());
    assertTrue(cIter.getValue().equals(new Vector(0, 1)));
    assertFalse(cIter.hasNext());
    assertThrows(() => cIter.nextValue());
    j = cList.store(new Vector(0.1, 1.1));
    assertEquals(1, j);
    assertEquals(2, cList.getSize());
    assertEquals(0, cList.getStartIndex());
    assertEquals(1, cList.getEndIndex());
    assertNotNull(endValue = cList.getEndValue());
    if (endValue)
        assertTrue(endValue.equals(new Vector(0.1, 1.1)));
    cIter = cList.getIterator();
    assertTrue(cIter.hasNext());
    cIter.nextValue();
    assertEquals(0, cIter.getIndex());
    assertTrue(cIter.getValue().equals(new Vector(0, 1)));
    assertTrue(cIter.hasNext());
    cIter.nextValue();
    assertEquals(1, cIter.getIndex());
    assertTrue(cIter.getValue().equals(new Vector(0.1, 1.1)));
    assertFalse(cIter.hasNext());
    assertThrows(() => cIter.nextValue());
    assertThrows(() => cList.getIterator(2));
    for (i = 2; i < 10; i++) {
        j = cList.store(new Vector(i * 0.1, (i < 5 ? 1 : 10) + i * 0.1));
        assertEquals(i, j);
    }
    assertEquals(i, cList.getSize());
    assertEquals(0, cList.getStartIndex());
    assertEquals(9, cList.getEndIndex());
    assertNotNull(endValue = cList.getEndValue());
    if (endValue)
        assertTrue(endValue.equals(new Vector(0.9, 10.9)));
    cIter = cList.getIterator();
    i = -1;
    while (cIter.hasNext()) {
        i++;
        cIter.nextValue();
        assertEquals(i, cIter.getIndex());
        assertTrue(cIter.getValue().equals(new Vector(i * 0.1, (i < 5 ? 1 : 10) + i * 0.1)));
    }
    assertEquals(9, i);
    assertThrows(() => cIter.nextValue());
    assertEquals(10, cList.getSize());
    assertEquals(0, cList.getStartIndex());
    assertEquals(9, cList.getEndIndex());
    j = cList.store(new Vector(1, 11));
    assertEquals(10, j);
    assertEquals(10, cList.getSize());
    assertEquals(1, cList.getStartIndex());
    assertEquals(10, cList.getEndIndex());
    cIter = cList.getIterator();
    i = 0;
    while (cIter.hasNext()) {
        i++;
        cIter.nextValue();
        assertEquals(i, cIter.getIndex());
        assertTrue(cIter.getValue().equals(new Vector(i * 0.1, (i < 5 ? 1 : 10) + i * 0.1)));
    }
    assertEquals(10, i);
    assertThrows(() => cIter.nextValue());
    for (i = 11; i < 20; i++) {
        j = cList.store(new Vector(i * 0.1, (i < 5 ? 1 : 10) + i * 0.1));
        assertEquals(i, j);
    }
    assertEquals(10, cList.getSize());
    assertEquals(10, cList.getStartIndex());
    assertEquals(19, cList.getEndIndex());
    cIter = cList.getIterator();
    i = 9;
    while (cIter.hasNext()) {
        i++;
        cIter.nextValue();
        assertEquals(i, cIter.getIndex());
        assertTrue(cIter.getValue().equals(new Vector(i * 0.1, (i < 5 ? 1 : 10) + i * 0.1)));
    }
    assertEquals(19, i);
    assertThrows(() => cIter.nextValue());
    j = cList.store(new Vector(2, 12));
    assertEquals(20, j);
    assertEquals(10, cList.getSize());
    assertEquals(11, cList.getStartIndex());
    assertEquals(20, cList.getEndIndex());
    cIter = cList.getIterator();
    i = 10;
    while (cIter.hasNext()) {
        i++;
        cIter.nextValue();
        assertEquals(i, cIter.getIndex());
        assertTrue(cIter.getValue().equals(new Vector(i * 0.1, (i < 5 ? 1 : 10) + i * 0.1)));
    }
    assertEquals(20, i);
    assertThrows(() => cIter.nextValue());
}
;
function testUtilCircularList2() {
    startTest(groupName + 'testUtilCircularList2');
    var cList = new CircularList(1000);
    cList.causeMaxIntError();
    var i;
    var j = -1;
    var e = assertThrows(() => {
        for (i = 0; i < 3000; i++) {
            j = cList.store(new Vector(i * 0.1, i * 10));
        }
    });
    assertEquals(CircularList.MAX_INDEX_ERROR, e);
    assertTrue(j >= Util.MAX_INTEGER - 2);
    cList.reset();
    assertEquals(0, cList.getSize());
    assertEquals(0, cList.getStartIndex());
    assertEquals(-1, cList.getEndIndex());
    assertNull(cList.getEndValue());
    for (i = 0; i < 3000; i++) {
        j = cList.store(new Vector(i * 0.1, i * 10));
    }
    assertEquals(2999, j);
    assertEquals(2000, cList.getStartIndex());
    assertEquals(2999, cList.getEndIndex());
    var cIter = cList.getIterator(2500);
    i = 2499;
    while (cIter.hasNext()) {
        i++;
        cIter.nextValue();
        assertEquals(i, cIter.getIndex());
        assertTrue(cIter.getValue().equals(new Vector(i * 0.1, i * 10)));
    }
    assertEquals(2999, i);
    assertThrows(() => cIter.nextValue());
}
;
function testUtilCircularList3() {
    startTest(groupName + 'testUtilCircularList3');
    var cList = new CircularList(100);
    var i, j, vec;
    for (i = 0; i < 90; i++) {
        j = cList.store(new Vector(i * 0.1, i * 10));
        vec = cList.getValue(j);
        assertTrue(vec.equals(new Vector(i * 0.1, i * 10)));
    }
    assertEquals(89, j);
    assertEquals(0, cList.getStartIndex());
    assertEquals(89, cList.getEndIndex());
    var cIter = cList.getIterator(cList.getEndIndex());
    i = 90;
    while (cIter.hasPrevious()) {
        i--;
        assertTrue(i >= 0);
        cIter.previousValue();
        assertEquals(i, cIter.getIndex());
        assertTrue(cIter.getValue().equals(new Vector(i * 0.1, i * 10)));
    }
    assertEquals(0, i);
    assertEquals(0, cIter.getIndex());
    assertFalse(cIter.hasPrevious());
    assertThrows(() => cIter.previousValue());
    for (i = 90; i < 110; i++) {
        j = cList.store(new Vector(i * 0.1, i * 10));
        vec = cList.getValue(j);
        assertTrue(vec.equals(new Vector(i * 0.1, i * 10)));
    }
    assertEquals(109, j);
    assertEquals(10, cList.getStartIndex());
    assertEquals(109, cList.getEndIndex());
    cIter = cList.getIterator(cList.getEndIndex());
    i = 110;
    while (cIter.hasPrevious()) {
        i--;
        assertTrue(i >= 10);
        cIter.previousValue();
        assertEquals(i, cIter.getIndex());
        assertTrue(cIter.getValue().equals(new Vector(i * 0.1, i * 10)));
    }
    assertEquals(10, i);
    assertEquals(10, cIter.getIndex());
    assertThrows(() => cIter.previousValue());
}
;
