import { ButtonControl } from '../../lab/controls/ButtonControl.js';
import { ChoiceControlBase } from '../../lab/controls/ChoiceControl.js';
import { DisplayAxes } from '../../lab/graph/DisplayAxes.js';
import { DisplayClock } from '../../lab/view/DisplayClock.js';
import { EasyScriptParser } from '../../lab/util/EasyScriptParser.js';
import { EnergyBarGraph } from '../../lab/graph/EnergyBarGraph.js';
import { GenericObserver, ParameterBoolean } from '../../lab/util/Observe.js';
import { GroupControl } from '../../lab/controls/GroupControl.js';
import { SimRunner } from '../../lab/app/SimRunner.js';
import { SimView } from '../../lab/view/SimView.js';
import { ToggleControl } from '../../lab/controls/ToggleControl.js';
import { Util } from '../../lab/util/Util.js';
export class CommonControls {
    constructor() {
        throw '';
    }
    ;
    static makeAxes(simView, bottomLeft_opt) {
        const axes = new DisplayAxes(simView.getSimRect());
        if (bottomLeft_opt) {
            axes.setXAxisAlignment("BOTTOM");
            axes.setYAxisAlignment("LEFT");
        }
        new GenericObserver(simView, evt => {
            if (evt.nameEquals(SimView.COORD_MAP_CHANGED)) {
                const r = simView.getCoordMap().screenToSimRect(simView.getScreenRect());
                axes.setSimRect(r);
            }
        }, 'resize axes');
        simView.getDisplayList().add(axes);
        return axes;
    }
    ;
    static makeBackgroundMenu(labCanvas) {
        const choices = [
            CommonControls.i18n.WHITE,
            CommonControls.i18n.BLACK,
            CommonControls.i18n.WHITE_WITH_TRAILS,
            CommonControls.i18n.BLACK_WITH_TRAILS,
            CommonControls.i18n.WHITE_WITH_LONG_TRAILS,
            CommonControls.i18n.BLACK_WITH_LONG_TRAILS
        ];
        let values = [
            CommonControls.en.WHITE,
            CommonControls.en.BLACK,
            CommonControls.en.WHITE_WITH_TRAILS,
            CommonControls.en.BLACK_WITH_TRAILS,
            CommonControls.en.WHITE_WITH_LONG_TRAILS,
            CommonControls.en.BLACK_WITH_LONG_TRAILS
        ];
        values = values.map(v => Util.toName(v));
        const longAlpha = CommonControls.LONG_TRAILS;
        const shortAlpha = CommonControls.SHORT_TRAILS;
        const getter = () => {
            const bg = labCanvas.getBackground();
            const alpha = labCanvas.getAlpha();
            if (bg == '') {
                return values[0];
            }
            else if (bg == 'black') {
                if (!Util.veryDifferent(alpha, 1)) {
                    return values[1];
                }
                else if (!Util.veryDifferent(alpha, shortAlpha)) {
                    return values[3];
                }
                else if (!Util.veryDifferent(alpha, longAlpha)) {
                    return values[5];
                }
            }
            else if (bg == 'white') {
                if (!Util.veryDifferent(alpha, 1)) {
                    return values[0];
                }
                else if (!Util.veryDifferent(alpha, shortAlpha)) {
                    return values[2];
                }
                else if (!Util.veryDifferent(alpha, longAlpha)) {
                    return values[4];
                }
            }
            return '';
        };
        const setter = (value) => {
            const idx = values.indexOf(value);
            switch (idx) {
                case 0:
                    labCanvas.setBackground('');
                    labCanvas.setAlpha(1);
                    break;
                case 1:
                    labCanvas.setBackground('black');
                    labCanvas.setAlpha(1);
                    break;
                case 2:
                    labCanvas.setBackground('white');
                    labCanvas.setAlpha(shortAlpha);
                    break;
                case 3:
                    labCanvas.setBackground('black');
                    labCanvas.setAlpha(shortAlpha);
                    break;
                case 4:
                    labCanvas.setBackground('white');
                    labCanvas.setAlpha(longAlpha);
                    break;
                case 5:
                    labCanvas.setBackground('black');
                    labCanvas.setAlpha(longAlpha);
                    break;
                default: throw '';
            }
        };
        const menu = new ChoiceControlBase(choices, values, getter, setter, CommonControls.i18n.BACKGROUND);
        labCanvas.addObserver(menu);
        return menu;
    }
    ;
    static makePanZoomControls(simView, overlay = true, resetFunc) {
        const imagesPath = Util.IMAGES_DIR + '/';
        const resetRect = simView.getSimRect();
        if (resetFunc === undefined) {
            resetFunc = () => simView.setSimRect(resetRect);
        }
        const debug = false;
        const sz = 30;
        const up_div = document.createElement('div');
        if (debug)
            up_div.style.border = 'dashed red thin';
        up_div.style.width = (sz * 3.2) + 'px';
        let img = Util.createImage(imagesPath + 'up_gray.png', sz);
        let bc = new ButtonControl('up', () => simView.panUp(), img);
        bc.repeatDelay = 100;
        up_div.appendChild(bc.getElement());
        const mid_div = document.createElement('div');
        if (debug)
            mid_div.style.border = 'dashed red thin';
        mid_div.style.width = up_div.style.width;
        img = Util.createImage(imagesPath + 'backward_gray.png', sz);
        bc = new ButtonControl('left', () => simView.panLeft(), img);
        bc.repeatDelay = 100;
        mid_div.appendChild(bc.getElement());
        img = Util.createImage(imagesPath + 'target_gray.png', sz);
        bc = new ButtonControl('reset', resetFunc, img);
        mid_div.appendChild(bc.getElement());
        img = Util.createImage(imagesPath + 'forward_gray.png', sz);
        bc = new ButtonControl('right', () => simView.panRight(), img);
        bc.repeatDelay = 100;
        mid_div.appendChild(bc.getElement());
        const down_div = document.createElement('div');
        img = Util.createImage(imagesPath + 'down_gray.png', sz);
        bc = new ButtonControl('down', () => simView.panDown(), img);
        bc.repeatDelay = 100;
        down_div.appendChild(bc.getElement());
        if (debug)
            down_div.style.border = 'dashed red thin';
        down_div.style.width = up_div.style.width;
        const pan_div = document.createElement('div');
        pan_div.appendChild(up_div);
        pan_div.appendChild(mid_div);
        pan_div.appendChild(down_div);
        if (debug)
            pan_div.style.border = 'dashed green thin';
        pan_div.style.textAlign = 'center';
        const zoom_div = document.createElement('div');
        img = Util.createImage(imagesPath + 'plus_gray.png', sz);
        bc = new ButtonControl('zoomIn', () => simView.zoomIn(), img);
        bc.repeatDelay = 100;
        zoom_div.appendChild(bc.getElement());
        zoom_div.appendChild(document.createElement('BR'));
        img = Util.createImage(imagesPath + 'minus_gray.png', sz);
        bc = new ButtonControl('zoomOut', () => simView.zoomOut(), img);
        bc.repeatDelay = 100;
        zoom_div.appendChild(bc.getElement());
        zoom_div.style.cssFloat = 'right';
        zoom_div.style.marginTop = (sz * 0.6) + 'px';
        if (debug)
            zoom_div.style.border = 'dashed blue thin';
        const panzoom_div = document.createElement('div');
        if (overlay) {
            panzoom_div.style.position = 'absolute';
            panzoom_div.style.left = '5%';
            panzoom_div.style.top = '5%';
            panzoom_div.style.width = (sz * 4.2) + 'px';
        }
        panzoom_div.appendChild(zoom_div);
        panzoom_div.appendChild(pan_div);
        if (debug)
            panzoom_div.style.border = 'solid gray thin';
        panzoom_div.style.display = 'none';
        return panzoom_div;
    }
    ;
    static makePlaybackControls(simrun, opt_overlay) {
        const imagesPath = Util.IMAGES_DIR + '/';
        const timer_div = document.createElement('div');
        timer_div.style.width = 'max-content';
        if (opt_overlay) {
            timer_div.style.position = 'absolute';
            timer_div.style.left = '0';
            timer_div.style.bottom = '0';
            timer_div.style.opacity = '0.5';
        }
        const sz = 30;
        let img = Util.createImage(imagesPath + 'restart.png', sz);
        const bc1 = new ButtonControl(SimRunner.i18n.RESTART, () => simrun.reset(), img);
        timer_div.appendChild(bc1.getElement());
        img = Util.createImage(imagesPath + 'forward.png', sz);
        const img2 = Util.createImage(imagesPath + 'pause.png', sz);
        const tc = new ToggleControl(simrun.getParameterBoolean(SimRunner.en.RUNNING), img, img2);
        timer_div.appendChild(tc.getElement());
        img = Util.createImage(imagesPath + 'next.png', sz);
        const bc2 = new ButtonControl(SimRunner.i18n.STEP, () => simrun.step(), img);
        bc2.repeatDelay = 100;
        timer_div.appendChild(bc2.getElement());
        const gc = new GroupControl('playback', timer_div, [bc1, tc, bc2]);
        return gc;
    }
    ;
    static makeEasyScript(subjects, dependent, simRun, terminal) {
        const easyScript = new EasyScriptParser(subjects, dependent);
        easyScript.addCommand('reset', () => simRun.reset(), 'sets simulation to initial conditions');
        easyScript.addCommand('save', () => simRun.save(), 'saves simulation initial conditions');
        easyScript.addCommand('step', () => simRun.step(), 'advance simulation by a small time increment');
        easyScript.addCommand('pause', () => simRun.pause(), 'pause simulation');
        easyScript.addCommand('resume', () => simRun.resume(), 'resume simulation');
        terminal.setParser(easyScript);
        return easyScript;
    }
    ;
    static makeShowClockParam(displayClock, targetView, subject) {
        const displayList = targetView.getDisplayList();
        const pb = new ParameterBoolean(subject, DisplayClock.en.SHOW_CLOCK, DisplayClock.i18n.SHOW_CLOCK, () => displayList.contains(displayClock), (value) => {
            if (value) {
                displayList.add(displayClock);
            }
            else {
                displayList.remove(displayClock);
            }
            subject.broadcastParameter(DisplayClock.en.SHOW_CLOCK);
        });
        subject.addParameter(pb);
        new GenericObserver(displayList, event => {
            if (event.getValue() == displayClock) {
                subject.broadcastParameter(DisplayClock.en.SHOW_CLOCK);
            }
        }, 'broadcast show clock parameter');
        return pb;
    }
    ;
    static makeShowEnergyParam(energyGraph, targetView, subject, opt_name, opt_i18n_name) {
        const paramName = opt_name ?? EnergyBarGraph.en.SHOW_ENERGY;
        const i18nName = opt_i18n_name ?? EnergyBarGraph.i18n.SHOW_ENERGY;
        const r = targetView.getCoordMap().screenToSimRect(targetView.getScreenRect());
        energyGraph.setVisibleArea(r);
        const displayList = targetView.getDisplayList();
        const pb = new ParameterBoolean(subject, paramName, i18nName, () => displayList.contains(energyGraph), (value) => {
            if (value) {
                displayList.add(energyGraph);
            }
            else {
                displayList.remove(energyGraph);
            }
            subject.broadcastParameter(paramName);
        });
        subject.addParameter(pb);
        new GenericObserver(displayList, event => {
            if (event.getValue() == energyGraph) {
                subject.broadcastParameter(paramName);
            }
        }, 'broadcast show energy parameter');
        return pb;
    }
    ;
    static makeShowPanZoomParam(panZoomDiv, subject) {
        const pb = new ParameterBoolean(subject, CommonControls.en.PAN_ZOOM, CommonControls.i18n.PAN_ZOOM, () => panZoomDiv.style.display != 'none', (value) => {
            if (value) {
                panZoomDiv.style.display = 'block';
            }
            else {
                panZoomDiv.style.display = 'none';
            }
            subject.broadcastParameter(CommonControls.en.PAN_ZOOM);
        });
        subject.addParameter(pb);
        return pb;
    }
    ;
    static makeURLScriptButton(easyScript, simRun) {
        const copyURL = () => {
            const u = easyScript.scriptURL();
            let p = EasyScriptParser.i18n.PROMPT_URL;
            if (u.length > 2048) {
                p = p + '  ' + EasyScriptParser.i18n.WARN_URL_2048;
            }
            const firing = simRun.getFiring();
            if (firing) {
                simRun.stopFiring();
            }
            window.prompt(p, u);
            if (firing) {
                simRun.startFiring();
            }
        };
        return new ButtonControl(EasyScriptParser.i18n.URL_SCRIPT, copyURL);
    }
    ;
}
CommonControls.SHORT_TRAILS = 0.1;
CommonControls.LONG_TRAILS = 0.05;
CommonControls.en = {
    PAN_ZOOM: 'pan-zoom',
    BACKGROUND: 'background',
    WHITE: 'white',
    BLACK: 'black',
    WHITE_WITH_TRAILS: 'white with trails',
    BLACK_WITH_TRAILS: 'black with trails',
    WHITE_WITH_LONG_TRAILS: 'white with long trails',
    BLACK_WITH_LONG_TRAILS: 'black with long trails'
};
CommonControls.de_strings = {
    PAN_ZOOM: 'pan-zoom',
    BACKGROUND: 'Hintergrund',
    WHITE: 'weiss',
    BLACK: 'schwarz',
    WHITE_WITH_TRAILS: 'weiss mit Pfade',
    BLACK_WITH_TRAILS: 'schwarz mit Pfade',
    WHITE_WITH_LONG_TRAILS: 'weiss mit lange Pfade',
    BLACK_WITH_LONG_TRAILS: 'schwarz mit lange Pfade'
};
CommonControls.i18n = Util.LOCALE === 'de' ? CommonControls.de_strings : CommonControls.en;
Util.defineGlobal('sims$common$CommonControls', CommonControls);
