import { AbstractSubject } from '../../lab/util/AbstractSubject.js';
import { AutoScale } from '../../lab/graph/AutoScale.js';
import { ButtonControl } from '../../lab/controls/ButtonControl.js';
import { CheckBoxControl } from '../../lab/controls/CheckBoxControl.js';
import { ChoiceControl } from '../../lab/controls/ChoiceControl.js';
import { CommonControls } from './CommonControls.js';
import { DisplayGraph } from '../../lab/graph/DisplayGraph.js';
import { DoubleRect } from '../../lab/util/DoubleRect.js';
import { GenericEvent, GenericObserver } from '../../lab/util/Observe.js';
import { GenericMemo } from '../../lab/util/Memo.js';
import { GraphColorChoices, GraphColorValues } from '../../lab/graph/GraphColor.js';
import { GraphLine } from '../../lab/graph/GraphLine.js';
import { NumericControl } from '../../lab/controls/NumericControl.js';
import { SimController } from '../../lab/app/SimController.js';
import { SimView } from '../../lab/view/SimView.js';
import { Util } from '../../lab/util/Util.js';
export class StandardGraph1 extends AbstractSubject {
    constructor(varsList, graphCanvas, div_controls, div_graph, simRun, displayStyle) {
        super('GRAPH_LAYOUT');
        this.displayStyle = displayStyle || 'block';
        this.canvas = graphCanvas;
        simRun.addCanvas(graphCanvas);
        this.view = new SimView('X_Y_GRAPH_VIEW', new DoubleRect(0, 0, 1, 1));
        this.view.setHorizAlign("FULL");
        this.view.setVerticalAlign("FULL");
        graphCanvas.addView(this.view);
        this.displayList = this.view.getDisplayList();
        this.line = new GraphLine('X_Y_GRAPH_LINE', varsList);
        this.line.setXVariable(0);
        this.line.setYVariable(1);
        this.line.setColor('lime');
        this.view.addMemo(this.line);
        this.axes = CommonControls.makeAxes(this.view);
        const updateAxes = (evt) => {
            if (evt.nameEquals(GraphLine.en.X_VARIABLE)) {
                this.axes.setHorizName(this.line.getXVarName());
            }
            if (evt.nameEquals(GraphLine.en.Y_VARIABLE)) {
                this.axes.setVerticalName(this.line.getYVarName());
            }
        };
        new GenericObserver(this.line, updateAxes, 'update axes names');
        updateAxes(new GenericEvent(this.line, GraphLine.i18n.X_VARIABLE));
        this.autoScale = new AutoScale('X_Y_AUTO_SCALE', this.line, this.view);
        this.autoScale.extraMargin = 0.05;
        this.displayGraph = new DisplayGraph(this.line);
        this.displayGraph.setScreenRect(this.view.getScreenRect());
        this.displayGraph.setUseBuffer(true);
        this.displayList.prepend(this.displayGraph);
        new GenericObserver(this.view, (evt) => {
            if (evt.nameEquals(SimView.SCREEN_RECT_CHANGED)) {
                this.displayGraph.setScreenRect(this.view.getScreenRect());
            }
        }, 'resize DisplayGraph');
        this.controls_ = [];
        this.div_controls = div_controls;
        let pn = this.line.getParameterNumber(GraphLine.en.Y_VARIABLE);
        this.addControl(new ChoiceControl(pn, 'Y:'));
        pn = this.line.getParameterNumber(GraphLine.en.X_VARIABLE);
        this.addControl(new ChoiceControl(pn, 'X:'));
        this.addControl(new ButtonControl(GraphLine.i18n.CLEAR_GRAPH, () => this.line.reset()));
        let ps = this.line.getParameterString(GraphLine.en.GRAPH_COLOR);
        this.addControl(new ChoiceControl(ps, undefined, GraphColorChoices(), GraphColorValues()));
        pn = this.line.getParameterNumber(GraphLine.en.LINE_WIDTH);
        this.addControl(new NumericControl(pn));
        ps = this.line.getParameterString(GraphLine.en.DRAWING_MODE);
        this.addControl(new ChoiceControl(ps));
        this.graphCtrl = new SimController(graphCanvas, null, { alt: false, control: false, meta: false, shift: false });
        const panzoom = CommonControls.makePanZoomControls(this.view, true, () => { this.autoScale.setActive(true); });
        div_graph.appendChild(panzoom);
        const pb = CommonControls.makeShowPanZoomParam(panzoom, this);
        this.addControl(new CheckBoxControl(pb));
        const timeIdx = this.line.getVarsList().timeIndex();
        simRun.addMemo(new GenericMemo(() => {
            if (this.line.getXVariable() == timeIdx) {
                const t = simRun.getClock().getTime();
                const tw = this.autoScale.getTimeWindow();
                this.displayGraph.setUseBuffer(t < tw);
            }
            else {
                this.displayGraph.setUseBuffer(true);
            }
        }, 'graph: use off-screen buffer when not time-scrolling'));
    }
    ;
    toString() {
        return this.toStringShort().slice(0, -1)
            + ', canvas: ' + this.canvas.toStringShort()
            + ', view: ' + this.view.toStringShort()
            + ', line: ' + this.line.toStringShort()
            + ', axes: ' + this.axes.toStringShort()
            + ', autoScale: ' + this.autoScale.toStringShort()
            + ', displayGraph: ' + this.displayGraph.toStringShort()
            + ', graphCtrl: ' + this.graphCtrl.toStringShort()
            + super.toString();
    }
    ;
    getClassName() {
        return 'StandardGraph1';
    }
    ;
    getSubjects() {
        return [this, this.line, this.view, this.autoScale];
    }
    ;
    addControl(control) {
        const element = control.getElement();
        element.style.display = this.displayStyle;
        this.div_controls.appendChild(element);
        this.controls_.push(control);
        return control;
    }
    ;
}
Util.defineGlobal('sims$common$StandardGraph1', StandardGraph1);
