import { AbstractSubject } from '../../lab/util/AbstractSubject.js';
import { AutoScale } from '../../lab/graph/AutoScale.js';
import { ButtonControl } from '../../lab/controls/ButtonControl.js';
import { CheckBoxControl } from '../../lab/controls/CheckBoxControl.js';
import { ChoiceControl } from '../../lab/controls/ChoiceControl.js';
import { CommonControls } from './CommonControls.js';
import { DisplayGraph } from '../../lab/graph/DisplayGraph.js';
import { DoubleRect } from '../../lab/util/DoubleRect.js';
import { GenericObserver } from '../../lab/util/Observe.js';
import { GenericMemo } from '../../lab/util/Memo.js';
import { GraphLine } from '../../lab/graph/GraphLine.js';
import { NumericControl } from '../../lab/controls/NumericControl.js';
import { SimController } from '../../lab/app/SimController.js';
import { SimRunner } from '../../lab/app/SimRunner.js';
import { SimView } from '../../lab/view/SimView.js';
import { Util } from '../../lab/util/Util.js';
export class TimeGraph1 extends AbstractSubject {
    constructor(varsList, graphCanvas, div_controls, div_graph, simRun, color1, color2, color3) {
        super('TIME_GRAPH_LAYOUT');
        this.controls_ = [];
        color1 = color1 ?? 'lime';
        color2 = color2 ?? 'red';
        color3 = color3 ?? 'blue';
        this.canvas = graphCanvas;
        simRun.addCanvas(graphCanvas);
        this.view = new SimView('TIME_GRAPH_VIEW', new DoubleRect(0, 0, 1, 1));
        this.view.setHorizAlign("FULL");
        this.view.setVerticalAlign("FULL");
        graphCanvas.addView(this.view);
        this.axes = CommonControls.makeAxes(this.view);
        this.line1 = new GraphLine('TIME_GRAPH_LINE_1', varsList);
        this.line1.setColor(color1);
        this.view.addMemo(this.line1);
        this.autoScale = new AutoScale('TIME_GRAPH_AUTO_SCALE', this.line1, this.view);
        this.autoScale.extraMargin = 0.05;
        this.displayGraph = new DisplayGraph(this.line1);
        this.displayGraph.setScreenRect(this.view.getScreenRect());
        this.view.getDisplayList().prepend(this.displayGraph);
        new GenericObserver(this.view, evt => {
            if (evt.nameEquals(SimView.SCREEN_RECT_CHANGED)) {
                this.displayGraph.setScreenRect(this.view.getScreenRect());
            }
        }, 'resize DisplayGraph');
        const timeIdx = this.line1.getVarsList().timeIndex();
        this.line1.setXVariable(timeIdx);
        this.line1.setYVariable(0);
        this.line2 = new GraphLine('TIME_GRAPH_LINE_2', varsList);
        this.autoScale.addGraphLine(this.line2);
        this.view.addMemo(this.line2);
        this.line2.setXVariable(timeIdx);
        this.line2.setYVariable(1);
        this.line2.setColor(color2);
        this.displayGraph.addGraphLine(this.line2);
        this.line3 = new GraphLine('TIME_GRAPH_LINE_3', varsList);
        this.autoScale.addGraphLine(this.line3);
        this.view.addMemo(this.line3);
        this.line3.setXVariable(timeIdx);
        this.line3.setYVariable(-1);
        this.line3.setColor(color3);
        this.displayGraph.addGraphLine(this.line3);
        this.div_controls = div_controls;
        let pn = this.line1.getParameterNumber(GraphLine.en.Y_VARIABLE);
        this.addControl(new ChoiceControl(pn, color1));
        pn = this.line2.getParameterNumber(GraphLine.en.Y_VARIABLE);
        this.addControl(new ChoiceControl(pn, color2));
        pn = this.line3.getParameterNumber(GraphLine.en.Y_VARIABLE);
        this.addControl(new ChoiceControl(pn, color3));
        pn = this.line1.getParameterNumber(GraphLine.en.X_VARIABLE);
        this.addControl(new ChoiceControl(pn, 'X:'));
        pn = this.autoScale.getParameterNumber(AutoScale.en.TIME_WINDOW);
        this.addControl(new NumericControl(pn));
        const bc = new ButtonControl(GraphLine.i18n.CLEAR_GRAPH, () => {
            this.line1.reset();
            this.line2.reset();
            this.line3.reset();
        });
        this.addControl(bc);
        this.line1Obs = new GenericObserver(this.line1, _evt => {
            this.line2.setXVariable(this.line1.getXVariable());
            this.line3.setXVariable(this.line1.getXVariable());
        }, 'ensure line2, line3 have same X variable as line1');
        this.graphCtrl = new SimController(graphCanvas, null, { alt: false, control: false, meta: false, shift: false });
        this.view.setScaleTogether(false);
        new GenericObserver(simRun, evt => {
            if (evt.nameEquals(SimRunner.RESET)) {
                const vw = this.view.getWidth();
                this.view.setCenterX(vw / 2);
                this.autoScale.setActive(true);
            }
        }, 'TimeGraph1: go to time zero on reset');
        simRun.addMemo(new GenericMemo(() => {
            if (this.line1.getXVariable() == timeIdx) {
                const t = simRun.getClock().getTime();
                const tw = this.autoScale.getTimeWindow();
                this.displayGraph.setUseBuffer(t < tw);
            }
            else {
                this.displayGraph.setUseBuffer(true);
            }
        }, 'time graph: use off-screen buffer when not time-scrolling'));
        const panzoom = CommonControls.makePanZoomControls(this.view, true, () => this.autoScale.setActive(true));
        div_graph.appendChild(panzoom);
        const pb = CommonControls.makeShowPanZoomParam(panzoom, this);
        this.addControl(new CheckBoxControl(pb));
    }
    ;
    toString() {
        return this.toStringShort().slice(0, -1)
            + ', canvas: ' + this.canvas.toStringShort()
            + ', view: ' + this.view.toStringShort()
            + ', line1: ' + this.line1.toStringShort()
            + ', line2: ' + this.line2.toStringShort()
            + ', line3: ' + this.line3.toStringShort()
            + ', axes: ' + this.axes.toStringShort()
            + ', autoScale: ' + this.autoScale.toStringShort()
            + ', displayGraph: ' + this.displayGraph.toStringShort()
            + ', graphCtrl: ' + this.graphCtrl.toStringShort()
            + super.toString();
    }
    ;
    getClassName() {
        return 'TimeGraph1';
    }
    ;
    addControl(control) {
        const element = control.getElement();
        element.style.display = 'block';
        this.div_controls.appendChild(element);
        this.controls_.push(control);
        return control;
    }
    ;
    getSubjects() {
        return [this, this.line1, this.line2, this.line3, this.view, this.autoScale];
    }
    ;
}
Util.defineGlobal('sims$common$TimeGraph1', TimeGraph1);
