import { DoubleRect } from '../../lab/util/DoubleRect.js';
import { JointUtil } from '../../lab/engine2D/JointUtil.js';
import { Shapes } from '../../lab/engine2D/Shapes.js';
import { Vector } from '../../lab/util/Vector.js';
import { Util } from '../../lab/util/Util.js';
export class ChainConfig {
    constructor() {
        throw '';
    }
    ;
    static makeChain(sim, options) {
        const joint1X = 0.5 * options.blockWidth;
        const joint1Y = 0.15 * options.blockHeight;
        const joint2X = 0.5 * options.blockWidth;
        const joint2Y = 0.85 * options.blockHeight;
        let body_angle = -180;
        const links = [];
        let bodyi;
        for (let i = 0; i < options.numLinks; i++) {
            bodyi = Shapes.makeBlock2(options.blockWidth, options.blockHeight, ChainConfig.en.CHAIN + '-' + i, ChainConfig.i18n.CHAIN + '-' + i);
            links.push(bodyi);
            sim.addBody(bodyi);
            body_angle += 180 / (options.numLinks + 1);
            bodyi.setAngle(Math.PI * body_angle / 180);
            if (i == 0) {
                bodyi.alignTo(new Vector(joint1X, joint1Y), new Vector(options.wallPivotX, options.wallPivotY), Math.PI * body_angle / 180);
            }
        }
        for (let i = 0; i < options.numLinks; i++) {
            if (i == 0 && options.fixedLeft) {
                JointUtil.attachFixedPoint(sim, links[i], new Vector(joint1X, joint1Y), 0);
            }
            if (i > 0) {
                JointUtil.attachRigidBody(sim, links[i - 1], new Vector(joint2X, joint2Y), links[i], new Vector(joint1X, joint1Y), 0);
            }
            if (options.fixedRight && i == options.numLinks - 1) {
                JointUtil.attachFixedPoint(sim, links[i], new Vector(joint2X, joint2Y), 0);
            }
        }
        sim.alignConnectors();
        let r = DoubleRect.EMPTY_RECT;
        for (let i = 0; i < options.numLinks; i++) {
            const body = links[i];
            body.setZeroEnergyLevel();
            r = r.union(body.getBoundsWorld());
        }
        return r;
    }
    ;
}
ChainConfig.en = {
    NUM_LINKS: 'chain links',
    WALLS: 'walls',
    EXTRA_BODY: 'extra body',
    FIXED_LEFT: 'fixed point left',
    FIXED_RIGHT: 'fixed point right',
    FIXED_LEFT_X: 'fixed point left X',
    FIXED_LEFT_Y: 'fixed point left Y',
    BLOCK_LENGTH: 'block length',
    BLOCK_WIDTH: 'block width',
    CHAIN: 'chain',
    WALL_WIDTH: 'wall width'
};
ChainConfig.de_strings = {
    NUM_LINKS: 'Kettenglieder',
    WALLS: 'Wände',
    EXTRA_BODY: 'extra Körper',
    FIXED_LEFT: 'Fixpunkt links',
    FIXED_RIGHT: 'Fixpunkt rechts',
    FIXED_LEFT_X: 'Fixpunkt links X',
    FIXED_LEFT_Y: 'Fixpunkt links Y',
    BLOCK_LENGTH: 'Blocklänge',
    BLOCK_WIDTH: 'Blockbreite',
    CHAIN: 'Kette',
    WALL_WIDTH: 'Wand breite'
};
ChainConfig.i18n = Util.LOCALE === 'de' ? ChainConfig.de_strings : ChainConfig.en;
Util.defineGlobal('sims$engine2D$ChainConfig', ChainConfig);
