import { AffineTransform } from '../../lab/util/AffineTransform.js';
import { CollisionAdvance } from '../../lab/model/CollisionAdvance.js';
import { CommonControls } from '../common/CommonControls.js';
import { ContactSim } from '../../lab/engine2D/ContactSim.js';
import { DampingLaw } from '../../lab/model/DampingLaw.js';
import { DisplayShape } from '../../lab/view/DisplayShape.js';
import { DoubleRect } from '../../lab/util/DoubleRect.js';
import { Engine2DApp } from './Engine2DApp.js';
import { GravityLaw } from '../../lab/model/GravityLaw.js';
import { JointUtil } from '../../lab/engine2D/JointUtil.js';
import { NumericControl } from '../../lab/controls/NumericControl.js';
import { Shapes } from '../../lab/engine2D/Shapes.js';
import { SixThrusters } from './SixThrusters.js';
import { Util } from '../../lab/util/Util.js';
import { Vector } from '../../lab/util/Vector.js';
import { Walls } from '../../lab/engine2D/Walls.js';
export class DoublePendulum2App extends Engine2DApp {
    constructor(elem_ids) {
        const simRect = new DoubleRect(-6, -6, 6, 6);
        const sim = new ContactSim();
        const advance = new CollisionAdvance(sim);
        super(elem_ids, simRect, sim, advance);
        this.layout.getSimCanvas().setBackground('black');
        this.layout.getSimCanvas().setAlpha(CommonControls.SHORT_TRAILS);
        this.sim.setShowForces(false);
        this.dampingLaw = new DampingLaw(0, 0.15, this.simList);
        this.sim.addForceLaw(this.dampingLaw);
        this.gravityLaw = new GravityLaw(8, this.simList);
        this.sim.addForceLaw(this.gravityLaw);
        this.block1 = Shapes.makeBlock(1, 3, DoublePendulum2App.en.BLOCK + 1, DoublePendulum2App.i18n.BLOCK + 1);
        this.block1.setPosition(new Vector(-1, -1), Math.PI / 4);
        this.block2 = Shapes.makeBlock(1, 3, DoublePendulum2App.en.BLOCK + 2, DoublePendulum2App.i18n.BLOCK + 2);
        this.block2.setPosition(new Vector(0, 0), 0);
        this.block3 = Shapes.makeBlock(1, 3, DoublePendulum2App.en.BLOCK + 3, DoublePendulum2App.i18n.BLOCK + 3);
        this.block3.setPosition(new Vector(-4, -4), Math.PI / 2);
        this.protoBlock = new DisplayShape();
        this.protoBlock.setStrokeStyle('lightGray');
        this.protoBlock.setFillStyle('');
        this.protoBlock.setThickness(3);
        const b1 = new DisplayShape(this.block1, this.protoBlock);
        this.displayList.add(b1);
        this.sim.addBody(this.block1);
        const b2 = new DisplayShape(this.block2, this.protoBlock);
        this.displayList.add(b2);
        this.sim.addBody(this.block2);
        const b3 = new DisplayShape(this.block3, this.protoBlock);
        b3.setStrokeStyle('orange');
        this.displayList.add(b3);
        this.sim.addBody(this.block3);
        const img = document.getElementById('tipper');
        b3.setImage(img);
        let at = AffineTransform.IDENTITY;
        at = at.scale(2.8, 2.8);
        at = at.translate(27, 20);
        at = at.rotate(Math.PI / 2);
        b3.setImageAT(at);
        b3.setImageClip(false);
        b3.setNameFont('');
        const cg = this.layout.getSimCanvas().getContext().createLinearGradient(-1, -1, 1, 1);
        cg.addColorStop(0, '#87CEFA');
        cg.addColorStop(1, 'white');
        b2.setFillStyle(cg);
        b2.setNameColor('gray');
        b2.setNameFont('12pt sans-serif');
        b2.setNameRotate(Math.PI / 2);
        b1.setImageDraw(function (context) {
            const pat = context.createPattern(img, 'repeat');
            if (pat != null) {
                context.fillStyle = pat;
                context.fill();
            }
        });
        b1.setNameFont('');
        JointUtil.attachFixedPoint(this.sim, this.block2, new Vector(0, -1.0), 1);
        JointUtil.attachRigidBody(this.sim, this.block2, new Vector(0, 1.0), this.block1, new Vector(0, 1.0), 0);
        this.sim.alignConnectors();
        const zel = Walls.make2(this.sim, this.simView.getSimRect());
        this.gravityLaw.setZeroEnergyLevel(zel);
        this.sim.setElasticity(0.8);
        this.sim.saveInitialState();
        this.thrustForce1 = SixThrusters.make(1.0, this.block3);
        this.thrustForce2 = SixThrusters.make(1.0, this.block1);
        this.rbeh.setThrusters(this.thrustForce1, 'right');
        this.rbeh.setThrusters(this.thrustForce2, 'left');
        this.sim.addForceLaw(this.thrustForce1);
        this.sim.addForceLaw(this.thrustForce2);
        this.addPlaybackControls();
        let pn = this.gravityLaw.getParameterNumber(GravityLaw.en.GRAVITY);
        this.addControl(new NumericControl(pn));
        this.watchEnergyChange(pn);
        pn = this.dampingLaw.getParameterNumber(DampingLaw.en.DAMPING);
        this.addControl(new NumericControl(pn));
        this.addStandardControls();
        this.makeEasyScript();
        this.addURLScriptButton();
        this.graphSetup();
    }
    ;
    toString() {
        return this.toStringShort().slice(0, -1)
            + ', dampingLaw: ' + this.dampingLaw.toStringShort()
            + ', gravityLaw: ' + this.gravityLaw.toStringShort()
            + super.toString();
    }
    ;
    getClassName() {
        return 'DoublePendulum2App';
    }
    ;
    defineNames(myName) {
        super.defineNames(myName);
        this.terminal.addRegex('gravityLaw|dampingLaw', myName + '.');
    }
    ;
    getSubjects() {
        return super.getSubjects().concat(this.dampingLaw, this.gravityLaw);
    }
    ;
}
DoublePendulum2App.en = {
    BLOCK: 'block'
};
DoublePendulum2App.de_strings = {
    BLOCK: 'Block'
};
DoublePendulum2App.i18n = Util.LOCALE === 'de' ? DoublePendulum2App.de_strings : DoublePendulum2App.en;
Util.defineGlobal('sims$engine2D$DoublePendulum2App', DoublePendulum2App);
