import { AbstractSubject } from '../../lab/util/AbstractSubject.js';
import { ParameterNumber } from '../../lab/util/Observe.js';
import { RigidBodySim } from '../../lab/engine2D/RigidBodySim.js';
import { Util } from '../../lab/util/Util.js';
export class ElasticitySetter extends AbstractSubject {
    constructor(sim) {
        super('ELASTICITY_SETTER');
        this.lastValue_ = 0;
        this.sim_ = sim;
        sim.addObserver(this);
        let pn = new ParameterNumber(this, ElasticitySetter.en.ELASTICITY, ElasticitySetter.i18n.ELASTICITY, () => this.getElasticity(), a => this.setElasticity(a));
        pn.setSignifDigits(3);
        pn.setUpperLimit(1);
        pn.setLowerLimit(0);
        this.addParameter(pn);
    }
    ;
    toString() {
        return this.toStringShort().slice(0, -1)
            + ', sim: ' + this.sim_.toStringShort()
            + super.toString();
    }
    ;
    getClassName() {
        return 'ElasticitySetter';
    }
    ;
    getElasticity() {
        this.lastValue_ = this.getElasticity_();
        return this.lastValue_;
    }
    ;
    getElasticity_() {
        const bods = this.sim_.getBodies();
        if (bods.length == 0) {
            return this.lastValue_;
        }
        else {
            return bods.reduce((min, body) => Math.min(min, body.getElasticity()), 1);
        }
    }
    ;
    setElasticity(value) {
        this.sim_.getBodies().forEach(body => body.setElasticity(value));
        this.lastValue_ = value;
        this.broadcastParameter(ElasticitySetter.en.ELASTICITY);
    }
    ;
    observe(event) {
        if (event.getSubject() == this.sim_ && event.nameEquals(RigidBodySim.ELASTICITY_SET)) {
            const nowValue = this.getElasticity_();
            if (this.lastValue_ != nowValue) {
                this.lastValue_ = nowValue;
                this.broadcastParameter(ElasticitySetter.en.ELASTICITY);
            }
        }
    }
    ;
}
ElasticitySetter.en = {
    ELASTICITY: 'elasticity'
};
ElasticitySetter.de_strings = {
    ELASTICITY: 'Elastizität'
};
ElasticitySetter.i18n = Util.LOCALE === 'de' ? ElasticitySetter.de_strings : ElasticitySetter.en;
Util.defineGlobal('sims$engine2D$ElasticitySetter', ElasticitySetter);
