import { AbstractSubject } from '../../lab/util/AbstractSubject.js';
import { CheckBoxControl } from '../../lab/controls/CheckBoxControl.js';
import { ChoiceControl } from '../../lab/controls/ChoiceControl.js';
import { Clock } from '../../lab/util/Clock.js';
import { CommonControls } from '../common/CommonControls.js';
import { DiffEqSolverSubject } from '../../lab/model/DiffEqSolverSubject.js';
import { DisplayClock } from '../../lab/view/DisplayClock.js';
import { DoubleRect } from '../../lab/util/DoubleRect.js';
import { ElasticitySetter } from './ElasticitySetter.js';
import { EnergyBarGraph } from '../../lab/graph/EnergyBarGraph.js';
import { EnergyInfo } from '../../lab/model/EnergySystem.js';
import { GenericObserver } from '../../lab/util/Observe.js';
import { LabelControl } from '../../lab/controls/LabelControl.js';
import { NumericControl } from '../../lab/controls/NumericControl.js';
import { RigidBodyEventHandler } from '../../lab/app/RigidBodyEventHandler.js';
import { RigidBodyObserver } from './RigidBodyObserver.js';
import { RigidBodySim } from '../../lab/engine2D/RigidBodySim.js';
import { SimController } from '../../lab/app/SimController.js';
import { SimRunner } from '../../lab/app/SimRunner.js';
import { SimView } from '../../lab/view/SimView.js';
import { StandardGraph1 } from '../common/StandardGraph1.js';
import { TabLayout } from '../common/TabLayout.js';
import { TimeGraph1 } from '../common/TimeGraph1.js';
import { Util } from '../../lab/util/Util.js';
import { Vector } from '../../lab/util/Vector.js';
import { VarsHistory } from '../../lab/graph/VarsHistory.js';
import { FunctionVariable } from '../../lab/model/FunctionVariable.js';
import { GenericMemo } from '../../lab/util/Memo.js';
import { DisplayText } from '../../lab/view/DisplayText.js';
import { ClockTask } from '../../lab/util/Clock.js';
export class Engine2DApp extends AbstractSubject {
    constructor(elem_ids, simRect, sim, advance, opt_name) {
        Util.setErrorHandler();
        super(opt_name || 'APP');
        this.simRect = simRect;
        this.layout = this.makeLayout(elem_ids);
        this.terminal = this.layout.getTerminal();
        const simCanvas = this.layout.getSimCanvas();
        this.sim = sim;
        sim.setTerminal(this.terminal);
        sim.setShowForces(false);
        this.simList = sim.getSimList();
        this.varsList = sim.getVarsList();
        this.advance = advance;
        this.simView = new SimView('SIM_VIEW', this.simRect);
        this.displayList = this.simView.getDisplayList();
        simCanvas.addView(this.simView);
        this.statusView = new SimView('STATUS_VIEW', new DoubleRect(-10, -10, 10, 10));
        this.statusList = this.statusView.getDisplayList();
        simCanvas.addView(this.statusView);
        this.axes = CommonControls.makeAxes(this.simView, true);
        this.simRun = new SimRunner(advance);
        this.simRun.addCanvas(simCanvas);
        this.clock = this.simRun.getClock();
        this.rbeh = new RigidBodyEventHandler(sim, this.clock);
        this.simCtrl = new SimController(simCanvas, this.rbeh);
        this.simRun.addErrorObserver(this.simCtrl);
        this.rbo = new RigidBodyObserver(this.simList, this.displayList);
        this.elasticity = new ElasticitySetter(sim);
        this.energyGraph = new EnergyBarGraph(sim);
        this.showEnergyParam = CommonControls.makeShowEnergyParam(this.energyGraph, this.statusView, this);
        this.displayClock = new DisplayClock(() => sim.getTime(), () => this.clock.getRealTime(), 2, 2);
        this.displayClock.setPosition(new Vector(8, 4));
        this.showClockParam = CommonControls.makeShowClockParam(this.displayClock, this.statusView, this);
        const panzoom = CommonControls.makePanZoomControls(this.simView, true, () => this.simView.setSimRect(this.simRect));
        this.layout.getSimDiv().appendChild(panzoom);
        this.panZoomParam = CommonControls.makeShowPanZoomParam(panzoom, this);
        this.panZoomParam.setValue(false);
        this.diffEqSolver = new DiffEqSolverSubject(sim, sim, advance);
        this.graph = new StandardGraph1(sim.getVarsList(), this.layout.getGraphCanvas(), this.layout.getGraphControls(), this.layout.getGraphDiv(), this.simRun);
        this.timeGraph = new TimeGraph1(sim.getVarsList(), this.layout.getTimeGraphCanvas(), this.layout.getTimeGraphControls(), this.layout.getTimeGraphDiv(), this.simRun);
        this.easyScript;
        this.terminal.setAfterEval(() => this.sim.modifyObjects());
    }
    ;
    toString() {
        return ', sim: ' + this.sim.toStringShort()
            + ', elasticity: ' + this.elasticity.toStringShort()
            + ', simList: ' + this.simList.toStringShort()
            + ', simCtrl: ' + this.simCtrl.toStringShort()
            + ', advance: ' + this.advance
            + ', simRect: ' + this.simRect
            + ', simView: ' + this.simView.toStringShort()
            + ', statusView: ' + this.statusView.toStringShort()
            + ', axes: ' + this.axes.toStringShort()
            + ', simRun: ' + this.simRun.toStringShort()
            + ', clock: ' + this.clock.toStringShort()
            + ', displayClock: ' + this.displayClock.toStringShort()
            + ', energyGraph: ' + this.energyGraph.toStringShort()
            + ', rbeh: ' + this.rbeh
            + ', rbo: ' + this.rbo
            + ', graph: ' + this.graph
            + ', timeGraph: ' + this.timeGraph
            + ', layout: ' + this.layout
            + ', easyScript: ' + this.easyScript.toStringShort()
            + ', terminal: ' + this.terminal
            + super.toString();
    }
    ;
    defineNames(myName) {
        this.simRun.setAppName(myName);
        this.terminal.addToVars(myName);
        this.terminal.addRegex('advance|axes|clock|diffEqSolver|displayClock|energyGraph'
            + '|graph|layout|sim|simCtrl|simList|simRect|simRun|simView|statusView|timeGraph'
            + '|displayList|easyScript|terminal|statusList|elasticity|varsList|rbo', myName + '.');
        this.terminal.addRegex('simCanvas', myName + '.layout.');
        this.terminal.addRegex('CommonControls|StandardGraph1|TimeGraph1|TabLayout', 'sims$$common$$', false);
    }
    ;
    makeLayout(elem_ids) {
        const canvasWidth = 800;
        const canvasHeight = Math.round(canvasWidth * this.simRect.getHeight() /
            this.simRect.getWidth());
        return new TabLayout(elem_ids, canvasWidth, canvasHeight);
    }
    ;
    watchEnergyChange(parameter) {
        new GenericObserver(parameter.getSubject(), evt => {
            if (evt == parameter) {
                this.sim.modifyObjects();
                this.sim.getVarsList().incrSequence(2, 3);
            }
        }, 'record discontinuous energy changes');
    }
    ;
    getSubjects() {
        const subjects = [
            this,
            this.sim,
            this.elasticity,
            this.diffEqSolver,
            this.simRun,
            this.clock,
            this.simView,
            this.statusView,
            this.varsList
        ];
        return subjects.concat(this.layout.getSubjects(), this.graph.getSubjects(), this.timeGraph.getSubjects());
    }
    ;
    makeEasyScript(opt_dependent) {
        let dependent = [this.varsList];
        if (opt_dependent) {
            dependent = dependent.concat(opt_dependent);
        }
        this.easyScript = CommonControls.makeEasyScript(this.getSubjects(), dependent, this.simRun, this.terminal);
    }
    ;
    graphSetup(body) {
        if (!body) {
            body = this.sim.getBodies().find(bod => isFinite(bod.getMass()));
        }
        else {
            if (this.graph.line.getXVariable() < 0) {
                this.graph.line.setXVariable(body.getVarsIndex() + 0);
            }
            if (this.graph.line.getYVariable() < 0) {
                this.graph.line.setYVariable(body.getVarsIndex() + 2);
            }
        }
        this.timeGraph.line1.setYVariable(1);
        this.timeGraph.line2.setYVariable(2);
        this.timeGraph.line3.setYVariable(3);
    }
    ;
    addPlaybackControls() {
        this.addControl(CommonControls.makePlaybackControls(this.simRun));
    }
    ;
    addURLScriptButton() {
        this.addControl(CommonControls.makeURLScriptButton(this.easyScript, this.simRun));
        this.graph.addControl(CommonControls.makeURLScriptButton(this.easyScript, this.simRun));
        this.timeGraph.addControl(CommonControls.makeURLScriptButton(this.easyScript, this.simRun));
    }
    ;
    addStandardControls() {
        let pn = this.elasticity.getParameterNumber(ElasticitySetter.en.ELASTICITY);
        this.addControl(new NumericControl(pn));
        let pb = this.sim.getParameterBoolean(RigidBodySim.en.SHOW_FORCES);
        this.addControl(new CheckBoxControl(pb));
        this.addControl(new CheckBoxControl(this.showEnergyParam));
        this.addControl(new CheckBoxControl(this.showClockParam));
        this.addControl(new CheckBoxControl(this.panZoomParam));
        pn = this.simRun.getParameterNumber(SimRunner.en.TIME_STEP);
        this.addControl(new NumericControl(pn));
        pn = this.simRun.getClock().getParameterNumber(Clock.en.TIME_RATE);
        this.addControl(new NumericControl(pn));
        let ps = this.diffEqSolver.getParameterString(DiffEqSolverSubject.en.DIFF_EQ_SOLVER);
        this.addControl(new ChoiceControl(ps));
        pn = this.sim.getParameterNumber(EnergyInfo.en.PE_OFFSET);
        this.addControl(new NumericControl(pn));
        let bm = CommonControls.makeBackgroundMenu(this.layout.getSimCanvas());
        this.addControl(bm);
        if (0 == 1 && Util.DEBUG) {
            this.addControl(new LabelControl('compiled ' + Util.COMPILE_TIME));
        }
    }
    ;
    addControl(control) {
        return this.layout.addControl(control);
    }
    ;
    eval(script, output = true) {
        return this.terminal.eval(script, output);
    }
    ;
    static loadClass() {
        var f = FunctionVariable.toString;
        f = VarsHistory.toString;
        f = GenericMemo.toString;
        f = DisplayText.toString;
        f = ClockTask.toString;
    }
    ;
    setup() {
        this.clock.resume();
        this.terminal.parseURL();
        this.sim.saveInitialState();
        this.sim.modifyObjects();
        this.simRun.memorize();
    }
    ;
    start() {
        this.simRun.startFiring();
    }
    ;
}
Util.defineGlobal('sims$engine2D$Engine2DApp', Engine2DApp);
