import { CollisionAdvance } from '../../lab/model/CollisionAdvance.js';
import { ContactSim } from '../../lab/engine2D/ContactSim.js';
import { DoubleRect } from '../../lab/util/DoubleRect.js';
import { Engine2DApp } from './Engine2DApp.js';
import { Shapes } from '../../lab/engine2D/Shapes.js';
import { Util } from '../../lab/util/Util.js';
import { Vector } from '../../lab/util/Vector.js';
import { Walls } from '../../lab/engine2D/Walls.js';
export class FastBallApp extends Engine2DApp {
    constructor(elem_ids) {
        const simRect = new DoubleRect(-6.5, -6.5, 6.5, 6.5);
        const sim = new ContactSim();
        const advance = new CollisionAdvance(sim);
        super(elem_ids, simRect, sim, advance);
        const p = Shapes.makeBall(0.2, FastBallApp.en.FAST_BALL, FastBallApp.i18n.FAST_BALL);
        p.setMass(0.1);
        p.setPosition(new Vector(-5, 0), 0);
        p.setVelocity(new Vector(200, 153), 0);
        this.sim.addBody(p);
        this.displayList.findShape(p).setFillStyle('green');
        this.sim.setElasticity(0.9);
        this.rbo.protoFixedPolygon.setFillStyle('black');
        Walls.make(this.sim, 12.0, 12.0, 0.01);
        this.addPlaybackControls();
        this.addStandardControls();
        this.makeEasyScript();
        this.addURLScriptButton();
        this.graphSetup();
    }
    ;
    toString() {
        return this.toStringShort().slice(0, -1)
            + super.toString();
    }
    ;
    getClassName() {
        return 'FastBallApp';
    }
    ;
}
FastBallApp.en = {
    FAST_BALL: 'fast ball'
};
FastBallApp.de_strings = {
    FAST_BALL: 'schnell Ball'
};
FastBallApp.i18n = Util.LOCALE === 'de' ? FastBallApp.de_strings : FastBallApp.en;
Util.defineGlobal('sims$engine2D$FastBallApp', FastBallApp);
