import { ChoiceControl } from '../../lab/controls/ChoiceControl.js';
import { CollisionAdvance } from '../../lab/model/CollisionAdvance.js';
import { ContactSim } from '../../lab/engine2D/ContactSim.js';
import { DampingLaw } from '../../lab/model/DampingLaw.js';
import { DoubleRect } from '../../lab/util/DoubleRect.js';
import { Engine2DApp } from './Engine2DApp.js';
import { JointUtil } from '../../lab/engine2D/JointUtil.js';
import { NumericControl } from '../../lab/controls/NumericControl.js';
import { ParameterNumber, ParameterString } from '../../lab/util/Observe.js';
import { Polygon } from '../../lab/engine2D/Polygon.js';
import { RigidBodySim } from '../../lab/engine2D/RigidBodySim.js';
import { Shapes } from '../../lab/engine2D/Shapes.js';
import { Util } from '../../lab/util/Util.js';
import { Vector } from '../../lab/util/Vector.js';
import { Walls } from '../../lab/engine2D/Walls.js';
;
export class MultipleCollisionApp extends Engine2DApp {
    constructor(elem_ids, opt_name) {
        const w = 6;
        const h = 2;
        const simRect = new DoubleRect(-w, -h, w, h);
        const sim = new ContactSim();
        const advance = new CollisionAdvance(sim);
        super(elem_ids, simRect, sim, advance, opt_name);
        this.offset = 0;
        this.angle = 0;
        this.speed = 3;
        this.space_half_width = w;
        this.space_half_height = h;
        this.layout.getSimCanvas().setBackground('black');
        this.sim.setCollisionHandling("serial grouped lastpass");
        this.elasticity.setElasticity(1.0);
        this.sim.setShowForces(true);
        this.dampingLaw = new DampingLaw(0, 0.15, this.simList);
        this.shape = "SQUARE";
        const i18n = MultipleCollisionApp.i18n;
        const choices = [i18n.ONE_HITS_TWO,
            i18n.ONE_HITS_THREE,
            i18n.ONE_HITS_TWO_SEPARATE,
            i18n.ONE_HITS_ONE_ON_WALL,
            i18n.ONE_HITS_TWO_ON_WALL,
            i18n.TWO_HIT_ONE,
            i18n.TWO_HIT_ONE_ASYMMETRIC,
            i18n.ONE_HITS_ONE,
            i18n.ONE_HITS_ONE_ASYMMETRIC,
            i18n.ONE_HITS_WALL,
            i18n.ONE_HITS_CHAIN,
            i18n.ONE_HITS_CHAIN_PLUS_ONE,
            i18n.TWO_IN_BOX,
            i18n.ONE_HITS_TWO_IN_BOX,
            i18n.TWO_ON_WALL
        ];
        const en = MultipleCollisionApp.en;
        this.formations = [en.ONE_HITS_TWO,
            en.ONE_HITS_THREE,
            en.ONE_HITS_TWO_SEPARATE,
            en.ONE_HITS_ONE_ON_WALL,
            en.ONE_HITS_TWO_ON_WALL,
            en.TWO_HIT_ONE,
            en.TWO_HIT_ONE_ASYMMETRIC,
            en.ONE_HITS_ONE,
            en.ONE_HITS_ONE_ASYMMETRIC,
            en.ONE_HITS_WALL,
            en.ONE_HITS_CHAIN,
            en.ONE_HITS_CHAIN_PLUS_ONE,
            en.TWO_IN_BOX,
            en.ONE_HITS_TWO_IN_BOX,
            en.TWO_ON_WALL
        ];
        this.formations = this.formations.map(v => Util.toName(v));
        this.formation = this.formations[0];
        this.addPlaybackControls();
        let pn;
        let ps;
        this.addParameter(ps = new ParameterString(this, MultipleCollisionApp.en.FORMATION, MultipleCollisionApp.i18n.FORMATION, () => this.getFormation(), a => this.setFormation(a), choices, this.formations));
        this.addControl(new ChoiceControl(ps));
        this.addParameter(ps = new ParameterString(this, MultipleCollisionApp.en.SHAPE, MultipleCollisionApp.i18n.SHAPE, () => this.getShape(), a => this.setShape(a), [MultipleCollisionApp.i18n.SQUARE, MultipleCollisionApp.i18n.CIRCLE], ["SQUARE", "CIRCLE"]));
        this.addControl(new ChoiceControl(ps));
        this.addParameter(pn = new ParameterNumber(this, MultipleCollisionApp.en.OFFSET, MultipleCollisionApp.i18n.OFFSET, () => this.getOffset(), a => this.setOffset(a)));
        this.addControl(new NumericControl(pn));
        this.addParameter(pn = new ParameterNumber(this, MultipleCollisionApp.en.ANGLE, MultipleCollisionApp.i18n.ANGLE, () => this.getAngle(), a => this.setAngle(a)));
        pn.setLowerLimit(Number.NEGATIVE_INFINITY);
        this.addControl(new NumericControl(pn));
        this.addParameter(pn = new ParameterNumber(this, MultipleCollisionApp.en.SPEED, MultipleCollisionApp.i18n.SPEED, () => this.getSpeed(), a => this.setSpeed(a)));
        this.addControl(new NumericControl(pn));
        pn = this.dampingLaw.getParameterNumber(DampingLaw.en.DAMPING);
        this.addControl(new NumericControl(pn));
        ps = this.sim.getParameterString(RigidBodySim.en.COLLISION_HANDLING);
        this.addControl(new ChoiceControl(ps));
        this.addStandardControls();
        this.makeEasyScript();
        this.addURLScriptButton();
        this.config();
        this.graphSetup();
    }
    ;
    toString() {
        return this.toStringShort().slice(0, -1)
            + ', dampingLaw: ' + this.dampingLaw.toStringShort()
            + ', formation: ' + this.formation
            + ', shape: ' + this.shape
            + ', angle: ' + Util.NF(this.angle)
            + ', offset: ' + Util.NF(this.offset)
            + ', speed: ' + Util.NF(this.speed)
            + super.toString();
    }
    ;
    getClassName() {
        return 'MultipleCollisionApp';
    }
    ;
    defineNames(myName) {
        super.defineNames(myName);
        this.terminal.addRegex('dampingLaw', myName + '.');
        this.terminal.addRegex('MultipleCollisionApp|Engine2DApp', 'sims$engine2D$', false);
    }
    ;
    getSubjects() {
        return super.getSubjects().concat(this.dampingLaw);
    }
    ;
    makePuck() {
        if (this.shape == "SQUARE") {
            return Shapes.makeBlock(1, 1);
        }
        else if (this.shape == "CIRCLE") {
            return Shapes.makeBall(0.5);
        }
        else {
            throw 'unknown shape';
        }
    }
    ;
    addBody(body) {
        const c = MultipleCollisionApp.massToColor(body.getMass());
        this.sim.addBody(body);
        const ds = this.displayList.findShape(body);
        ds.setFillStyle(c);
        ds.setDrawCenterOfMass(true);
    }
    ;
    static massToColor(mass) {
        let logm = Math.LOG10E * Math.log((mass));
        if (logm < -1) {
            logm = -1;
        }
        else if (logm > 2) {
            logm = 2;
        }
        const rgb = Math.floor(0.5 + 100 + 43 * (-logm + 2));
        const s = rgb.toString();
        return 'rgb(' + s + ',' + s + ',' + s + ')';
    }
    ;
    config() {
        const elasticity = this.elasticity.getElasticity();
        this.sim.cleanSlate();
        Polygon.ID = 1;
        this.advance.reset();
        const distTol = this.sim.getDistanceTol();
        let body, body1, body2, body3, body4;
        const idx = this.formations.indexOf(this.formation);
        switch (idx) {
            case 0:
                body = this.makePuck();
                body.setPosition(new Vector(-5, 0), this.angle);
                body.setVelocity(new Vector(this.speed, 0), 0);
                this.addBody(body);
                body = this.makePuck();
                body.setPosition(new Vector(0, 0));
                this.addBody(body);
                body = this.makePuck();
                body.setPosition(new Vector(1 + distTol / 2 + this.offset, 0));
                this.addBody(body);
                break;
            case 1:
                body = this.makePuck();
                body.setPosition(new Vector(-5, 0), this.angle);
                body.setVelocity(new Vector(this.speed, 0), 0);
                this.addBody(body);
                body = this.makePuck();
                body.setPosition(new Vector(-1 - distTol / 2 + this.offset, 0));
                this.addBody(body);
                body = this.makePuck();
                body.setPosition(new Vector(0, 0));
                this.addBody(body);
                body = this.makePuck();
                body.setPosition(new Vector(1 + distTol / 2 + this.offset, 0));
                this.addBody(body);
                break;
            case 2:
                body = Shapes.makeBlock(1, 3);
                body.setMass(2);
                body.setPosition(new Vector(-5, 0), this.angle);
                body.setVelocity(new Vector(this.speed, 0), 0);
                this.addBody(body);
                body = this.makePuck();
                body.setPosition(new Vector(0, 1), 0);
                this.addBody(body);
                body = this.makePuck();
                body.setPosition(new Vector(0, -1), 0);
                this.addBody(body);
                break;
            case 3:
                body = this.makePuck();
                body.setMass(1000);
                body.setPosition(new Vector(0, 0), this.angle);
                body.setVelocity(new Vector(this.speed, 0), 0);
                this.addBody(body);
                body = this.makePuck();
                body.setPosition(new Vector(this.space_half_width - 0.5 - distTol / 2, 0), 0);
                this.addBody(body);
                break;
            case 4:
                body = this.makePuck();
                body.setPosition(new Vector(0, 0), this.angle);
                body.setVelocity(new Vector(this.speed, 0), 0);
                this.addBody(body);
                body = this.makePuck();
                body.setPosition(new Vector(this.space_half_width - 1.5 - distTol, 0), 0);
                this.addBody(body);
                body = this.makePuck();
                body.setPosition(new Vector(this.space_half_width - 0.5 - distTol / 2, 0), 0);
                this.addBody(body);
                break;
            case 5:
                body = this.makePuck();
                body.setPosition(new Vector(-5, 0), this.angle);
                body.setVelocity(new Vector(this.speed, 0), 0);
                this.addBody(body);
                body = this.makePuck();
                body.setPosition(new Vector(0, 0));
                this.addBody(body);
                body = this.makePuck();
                body.setPosition(new Vector(5, 0), this.angle);
                body.setVelocity(new Vector(-this.speed, 0), 0);
                this.addBody(body);
                break;
            case 6:
                body = this.makePuck();
                body.setPosition(new Vector(-5 + distTol / 2, 0), this.angle);
                body.setVelocity(new Vector(this.speed, 0), 0);
                this.addBody(body);
                body = this.makePuck();
                body.setPosition(new Vector(0, 0));
                this.addBody(body);
                body = this.makePuck();
                body.setPosition(new Vector(3, 0));
                body.setVelocity(new Vector(-this.speed / 2, 0), 0);
                this.addBody(body);
                break;
            case 7:
                body = this.makePuck();
                body.setPosition(new Vector(-5, 0), this.angle);
                body.setVelocity(new Vector(this.speed, 0), 0);
                this.addBody(body);
                body = this.makePuck();
                body.setPosition(new Vector(0, 0));
                this.addBody(body);
                break;
            case 8:
                body = this.makePuck();
                body.setPosition(new Vector(-5, 0), this.angle);
                body.setVelocity(new Vector(this.speed, 0), 0);
                this.addBody(body);
                body = this.makePuck();
                body.setPosition(new Vector(2.75, 0));
                body.setVelocity(new Vector(-this.speed / 2, 0), 0);
                this.addBody(body);
                break;
            case 9:
                body = this.makePuck();
                body.setPosition(new Vector(0, 0), this.angle);
                body.setVelocity(new Vector(this.speed, 0), 0);
                this.addBody(body);
                break;
            case 11:
                body = this.makePuck();
                body.setMass(2);
                body.setPosition(new Vector(1 + distTol / 2, 0));
                this.addBody(body);
            case 10:
                body = this.makePuck();
                body.setMass(2);
                body.setPosition(new Vector(-5, 0), this.angle);
                body.setVelocity(new Vector(this.speed, 0), 0);
                this.addBody(body);
                body3 = Shapes.makeBlock(2, 1);
                body3.setPosition(new Vector(-0.5, 0));
                body3.setMass(1.5);
                this.addBody(body3);
                body2 = Shapes.makeBlock(2, 1);
                body2.setPosition(new Vector(-2, 0));
                body2.setMass(0.5);
                this.sim.addBody(body2);
                this.displayList.findShape(body2).setFillStyle('rgb(240,240,240)');
                JointUtil.attachRigidBody(this.sim, body2, new Vector(0.75, 0), body3, new Vector(-0.75, 0), 0);
                this.sim.alignConnectors();
                break;
            case 12:
                body = Shapes.makeFrame(2 + 3 * distTol / 2 + 0.2, 1 + 2 * distTol / 2 + 0.2, 0.2);
                body.setPosition(new Vector(0, 0));
                this.addBody(body);
                body = Shapes.makeBall(0.5 - this.offset / 2);
                body.setPosition(new Vector(-0.5 - distTol / 4, 0));
                this.addBody(body);
                body = Shapes.makeBall(0.5 - this.offset / 2);
                body.setPosition(new Vector(0.5 + distTol / 4, 0));
                body.setVelocity(new Vector(this.speed, 0), 0);
                this.addBody(body);
                break;
            case 13:
                body1 = Shapes.makeFrame(2 + 3 * distTol / 2 + 0.2, 1 + 2 * distTol / 2 + 0.2, 0.2);
                body1.setPosition(new Vector(0, 0));
                this.addBody(body1);
                body2 = Shapes.makeBall(0.5 - this.offset / 2);
                body2.setPosition(new Vector(-0.5 - distTol / 4, 0));
                this.addBody(body2);
                body3 = Shapes.makeBall(0.5 - this.offset / 2);
                body3.setPosition(new Vector(0.5 + distTol / 4, 0));
                this.addBody(body3);
                body4 = Shapes.makeBall(0.5);
                body4.setPosition(new Vector(-5, 0));
                body4.setVelocity(new Vector(this.speed, 0), 0);
                this.addBody(body4);
                break;
            case 14:
                body = this.makePuck();
                body.setMass(1);
                body.setPosition(new Vector(this.space_half_width - 1.5 - distTol, 0), 0);
                this.addBody(body);
                body = this.makePuck();
                body.setPosition(new Vector(this.space_half_width - 0.5 - distTol / 2, 0), 0);
                body.setVelocity(new Vector(this.speed, 0), 0);
                this.addBody(body);
                break;
            default:
                throw '';
        }
        Walls.make(this.sim, 2 * this.space_half_width, 2 * this.space_half_height, 1.0);
        this.sim.setElasticity(elasticity);
        this.sim.addForceLaw(this.dampingLaw);
        this.dampingLaw.connect(this.sim.getSimList());
        this.sim.getVarsList().setTime(0);
        this.sim.saveInitialState();
        this.clock.setTime(this.sim.getTime());
        this.clock.setRealTime(this.sim.getTime());
        this.easyScript.update();
    }
    ;
    getFormation() {
        return this.formation;
    }
    ;
    setFormation(value) {
        value = Util.toName(value);
        if (this.formation != value) {
            if (!this.formations.includes(value)) {
                throw 'unknown formation: ' + value;
            }
            this.formation = value;
            this.config();
            this.broadcastParameter(MultipleCollisionApp.en.FORMATION);
        }
    }
    ;
    getOffset() {
        return this.offset;
    }
    ;
    setOffset(value) {
        this.offset = value;
        this.config();
        this.broadcastParameter(MultipleCollisionApp.en.OFFSET);
    }
    ;
    getSpeed() {
        return this.speed;
    }
    ;
    setSpeed(value) {
        this.speed = value;
        this.config();
        this.broadcastParameter(MultipleCollisionApp.en.SPEED);
    }
    ;
    getAngle() {
        return this.angle;
    }
    ;
    setAngle(value) {
        this.angle = value;
        this.config();
        this.broadcastParameter(MultipleCollisionApp.en.ANGLE);
    }
    ;
    getShape() {
        return this.shape;
    }
    ;
    setShape(value) {
        value = Util.toName(value);
        if (this.shape != value) {
            this.shape = value;
            this.config();
            this.broadcastParameter(MultipleCollisionApp.en.SHAPE);
        }
    }
    ;
}
MultipleCollisionApp.en = {
    FORMATION: 'formation',
    ONE_HITS_THREE: 'one hits three',
    ONE_HITS_TWO: 'one hits two',
    TWO_HIT_ONE: 'two hit one',
    ONE_HITS_ONE_ON_WALL: 'one hits one on wall',
    ONE_HITS_TWO_ON_WALL: 'one hits two on wall',
    ONE_HITS_TWO_SEPARATE: 'one hits two separate',
    TWO_HIT_ONE_ASYMMETRIC: 'two hit one asymmetric',
    ONE_HITS_ONE: 'one hits one',
    ONE_HITS_ONE_ASYMMETRIC: 'one hits one asymmetric',
    ONE_HITS_WALL: 'one hits wall',
    ONE_HITS_CHAIN: 'one hits chain',
    ONE_HITS_CHAIN_PLUS_ONE: 'one hits chain plus one',
    TWO_IN_BOX: 'two in box',
    ONE_HITS_TWO_IN_BOX: 'one hits two in box',
    TWO_ON_WALL: 'two on wall',
    ANGLE: 'angle',
    SHAPE: 'shape',
    CIRCLE: 'circle',
    SQUARE: 'square',
    OFFSET: 'offset',
    PUCK_TYPE: 'puck type',
    SPEED: 'speed',
    MASS: 'mass'
};
MultipleCollisionApp.de_strings = {
    FORMATION: 'Formation',
    ONE_HITS_THREE: 'eins schlägt drei',
    ONE_HITS_TWO: 'eins schlägt zwei',
    TWO_HIT_ONE: 'zwei schlagen eins',
    ONE_HITS_ONE_ON_WALL: 'eins schlägt eins an einer Mauer',
    ONE_HITS_TWO_ON_WALL: 'eins schlägt zwei an einer Mauer',
    ONE_HITS_TWO_SEPARATE: 'eins schlägt zwei getrennt',
    TWO_HIT_ONE_ASYMMETRIC: 'zwei schlagen eins asymmetrisch',
    ONE_HITS_ONE: 'eins schlägt eins',
    ONE_HITS_ONE_ASYMMETRIC: 'eins schlägt eins asymmetrisch',
    ONE_HITS_WALL: 'eins schlägt eine Mauer',
    ONE_HITS_CHAIN: 'eins schlägt eine Kette',
    ONE_HITS_CHAIN_PLUS_ONE: 'eins schlägt eine Kette+1',
    TWO_IN_BOX: 'zwei in einer Schachtel',
    ONE_HITS_TWO_IN_BOX: 'eins schlägt zwei in einer Schachtel',
    TWO_ON_WALL: 'zwei an der Mauer',
    ANGLE: 'Winkel',
    SHAPE: 'Form',
    CIRCLE: 'Kreis',
    SQUARE: 'Quadrat',
    OFFSET: 'Abstand',
    PUCK_TYPE: 'Scheiben Typ',
    SPEED: 'Geschwindigkeit',
    MASS: 'Masse'
};
MultipleCollisionApp.i18n = Util.LOCALE === 'de' ? MultipleCollisionApp.de_strings : MultipleCollisionApp.en;
Util.defineGlobal('sims$engine2D$MultipleCollisionApp', MultipleCollisionApp);
