import { ConcreteVertex } from '../../lab/engine2D/ConcreteVertex.js';
import { EdgeRange } from '../../lab/engine2D/EdgeSet.js';
import { GearsConfig } from './GearsConfig.js';
import { JointUtil } from '../../lab/engine2D/JointUtil.js';
import { Polygon } from '../../lab/engine2D/Polygon.js';
import { Util } from '../../lab/util/Util.js';
import { Vector } from '../../lab/util/Vector.js';
export class PendulumClockConfig {
    constructor() {
        throw '';
    }
    ;
    static makeAnchor(scale, anchorJoint, rodLength, bobRadius, startEdges) {
        const p = new Polygon(PendulumClockConfig.en.ANCHOR, PendulumClockConfig.i18n.ANCHOR);
        p.startPath(new ConcreteVertex(new Vector(61 * scale, 104 * scale)));
        let e = p.addStraightEdge(new Vector(8 * scale, 162 * scale), false);
        startEdges.push(e);
        p.addStraightEdge(new Vector(109 * scale, 197 * scale), true);
        p.addStraightEdge(new Vector(221 * scale, 146 * scale), true);
        p.addStraightEdge(new Vector(155 * scale, 103 * scale), false);
        p.addStraightEdge(new Vector(159 * scale, 127 * scale), true);
        p.addStraightEdge(new Vector(73 * scale, 141 * scale), false);
        p.addStraightEdge(new Vector(61 * scale, 104 * scale), false);
        p.closePath();
        const width = 0.05;
        const xo = anchorJoint.getX();
        const yo = anchorJoint.getY() - rodLength - bobRadius;
        p.startPath(new ConcreteVertex(new Vector(width + xo, bobRadius + yo)));
        e = p.addStraightEdge(new Vector(width + xo, rodLength + width + yo), true);
        startEdges.push(e);
        p.addStraightEdge(new Vector(-width + xo, rodLength + width + yo), true);
        p.addStraightEdge(new Vector(-width + xo, bobRadius + yo), false);
        p.addCircularEdge(new Vector(width + xo, bobRadius + yo), new Vector(xo, yo), false, true);
        p.closePath();
        p.finish();
        p.setCenterOfMass(new Vector(xo, yo));
        p.setDragPoints([new Vector(xo, yo)]);
        const r = Math.sqrt(width * width + bobRadius * bobRadius);
        p.setMomentAboutCM(r * r / 2);
        return p;
    }
    ;
    static makeEscapeWheel(scale, withGear, startEdges) {
        const p = new Polygon(PendulumClockConfig.en.ESCAPE_WHEEL, PendulumClockConfig.i18n.ESCAPE_WHEEL);
        const radius = 90 * scale;
        const depth = 14 * scale;
        const numTeeth = 24;
        const in_0 = new Vector(radius, 0);
        const out_0 = new Vector(radius + depth, 0);
        let in_p = in_0;
        let out_p = out_0;
        p.startPath(new ConcreteVertex(new Vector(in_p.getX(), in_p.getY())));
        for (let i = 1; i <= numTeeth; i++) {
            let outsideIsUp;
            if (Math.abs(in_p.getX() - out_p.getX()) < 1e-2)
                outsideIsUp = out_p.getY() > in_p.getY();
            else
                outsideIsUp = in_p.getX() > out_p.getX();
            const e = p.addStraightEdge(out_p, outsideIsUp);
            if (i == 1) {
                startEdges.push(e);
            }
            const angle = (Math.PI * 2 * i) / numTeeth;
            in_p = in_0.rotate(angle);
            if (Math.abs(in_p.getX() - out_p.getX()) < 1e-2)
                outsideIsUp = out_p.getY() < in_p.getY();
            else
                outsideIsUp = in_p.getX() < out_p.getX();
            p.addStraightEdge(in_p, outsideIsUp);
            out_p = out_0.rotate(angle);
        }
        p.closePath();
        if (withGear) {
            GearsConfig.addGear(p, radius * 0.8, 0.3, 36, 30, 30, startEdges);
        }
        p.finish();
        p.setCenterOfMass(Vector.ORIGIN);
        p.setDragPoints([new Vector(0, 2)]);
        const r = radius + depth / 2;
        p.setMomentAboutCM(r * r / 2);
        return p;
    }
    ;
    static makeClock(sim, pendulumLength, center) {
        const escapeEdges = [];
        const escapeWheel = PendulumClockConfig.makeEscapeWheel(0.03, false, escapeEdges);
        escapeWheel.setPosition(center, 0);
        escapeWheel.setMass(1);
        sim.addBody(escapeWheel);
        JointUtil.attachFixedPoint(sim, escapeWheel, Vector.ORIGIN, 1);
        const scale = 0.03;
        const anchorJoint = new Vector(109 * scale, 166 * scale);
        const anchorEdges = [];
        const anchor = PendulumClockConfig.makeAnchor(scale, anchorJoint, pendulumLength, 0.6, anchorEdges);
        Util.assert(anchorEdges.length >= 2);
        anchor.setMass(10);
        sim.addBody(anchor);
        const p = center.add(new Vector(0, 4.6));
        anchor.alignTo(anchorJoint, p, 0);
        JointUtil.attachFixedPoint(sim, anchor, anchorJoint, 0);
        anchor.setZeroEnergyLevel();
        anchor.alignTo(anchorJoint, p, 0.13);
        sim.alignConnectors();
        escapeWheel.setZeroEnergyLevel();
        escapeWheel.setNonCollideEdge(EdgeRange.fromEdge(anchorEdges[1]));
    }
    ;
    static makeClockWithGears(sim, pendulumLength, center) {
        const scale = 0.03;
        const anchorJoint = new Vector(109 * scale, 166 * scale);
        const anchorEdges = [];
        const anchor = PendulumClockConfig.makeAnchor(scale, anchorJoint, pendulumLength, 0.6, anchorEdges);
        Util.assert(anchorEdges.length >= 2);
        anchor.setMass(10);
        sim.addBody(anchor);
        let p = center.add(new Vector(0, 4.6));
        anchor.alignTo(anchorJoint, p, 0.0);
        JointUtil.attachFixedPoint(sim, anchor, anchorJoint, 0);
        anchor.setZeroEnergyLevel();
        anchor.alignTo(anchorJoint, p, 0.13);
        sim.alignConnectors();
        const r = scale * 90 * 0.8;
        const gear2Edges = [];
        const gear2 = GearsConfig.makeGear(r, gear2Edges, PendulumClockConfig.en.GEAR + 2, PendulumClockConfig.i18n.GEAR + 2);
        gear2.setMass(0.5);
        const tooth = 2 * Math.PI / 36;
        p = center.add(new Vector((2 * r) + 0.008 + 0.40, 0));
        gear2.setPosition(p, -tooth / 5 - 0.002);
        gear2.addNonCollide([anchor]);
        anchor.addNonCollide([gear2]);
        sim.addBody(gear2);
        JointUtil.attachFixedPoint(sim, gear2, new Vector(0, 0), 1);
        gear2.setZeroEnergyLevel();
        const escapeEdges = [];
        const escapeWheel = PendulumClockConfig.makeEscapeWheel(0.03, true, escapeEdges);
        Util.assert(escapeEdges.length >= 2);
        escapeWheel.setPosition(center, 0);
        escapeWheel.setMass(1);
        sim.addBody(escapeWheel);
        JointUtil.attachFixedPoint(sim, escapeWheel, Vector.ORIGIN, 1);
        escapeWheel.setZeroEnergyLevel();
        escapeWheel.setNonCollideEdge(EdgeRange.fromEdge(anchorEdges[1]));
        gear2.setNonCollideEdge(EdgeRange.fromEdge(escapeEdges[0]));
        anchor.setNonCollideEdge(EdgeRange.fromEdge(escapeEdges[1]));
    }
    ;
}
PendulumClockConfig.en = {
    ANCHOR: 'anchor',
    ESCAPE_WHEEL: 'escape wheel',
    EXTRA_BODY: 'extra body',
    PENDULUM_LENGTH: 'pendulum length',
    TURNING_FORCE: 'turning force',
    WITH_GEARS: 'with gears',
    GEAR: 'gear'
};
PendulumClockConfig.de_strings = {
    ANCHOR: 'Anker',
    ESCAPE_WHEEL: 'escape Rad',
    EXTRA_BODY: 'extra Körper',
    PENDULUM_LENGTH: 'Pendellänge',
    TURNING_FORCE: 'drehende Kraft',
    WITH_GEARS: 'mit Zahnrädern',
    GEAR: 'Zahnrad'
};
PendulumClockConfig.i18n = Util.LOCALE === 'de' ? PendulumClockConfig.de_strings : PendulumClockConfig.en;
Util.defineGlobal('sims$engine2D$PendulumClockConfig', PendulumClockConfig);
