import { JointUtil } from '../../lab/engine2D/JointUtil.js';
import { RandomLCG } from '../../lab/util/Random.js';
import { Shapes } from '../../lab/engine2D/Shapes.js';
import { Util } from '../../lab/util/Util.js';
import { Vector } from '../../lab/util/Vector.js';
import { Walls } from '../../lab/engine2D/Walls.js';
export class PileConfig {
    constructor() {
        throw '';
    }
    ;
    static makeVPit(sim, opt_offset) {
        const offset = opt_offset ?? 0;
        const p1 = Shapes.makeWall(15.3, 1, Shapes.TOP_EDGE, Walls.en.WALL_BOTTOM + '_' + PileConfig.en.LEFT, Walls.i18n.WALL_BOTTOM + '_' + PileConfig.i18n.LEFT);
        p1.setPosition(new Vector(-5, -5 + offset), -Math.PI / 4);
        p1.setMass(Infinity);
        sim.addBody(p1);
        const p2 = Shapes.makeWall(15.3, 1, Shapes.TOP_EDGE, Walls.en.WALL_BOTTOM + '_' + PileConfig.en.RIGHT, Walls.i18n.WALL_BOTTOM + '_' + PileConfig.i18n.RIGHT);
        p2.setPosition(new Vector(5, -5 + offset), Math.PI / 4);
        p2.setMass(Infinity);
        sim.addBody(p2);
        const p3 = Shapes.makeWall(1, 15, Shapes.LEFT_EDGE, Walls.en.WALL_RIGHT, Walls.i18n.WALL_RIGHT);
        p3.setPosition(new Vector(10.5, 7.5 + offset), 0);
        p3.setMass(Infinity);
        sim.addBody(p3);
        const p4 = Shapes.makeWall(1, 15, Shapes.RIGHT_EDGE, Walls.en.WALL_LEFT, Walls.i18n.WALL_LEFT);
        p4.setPosition(new Vector(-10.5, 7.5 + offset), 0);
        p4.setMass(Infinity);
        sim.addBody(p4);
        const walls = [p1, p2, p3, p4];
        walls.forEach(w => w.addNonCollide(walls));
        return p1.bodyToWorld(new Vector(15.3 / 2 - 1, 0.5)).getY();
    }
    ;
    static makeDoubleVPit(sim, opt_offset) {
        const offset = opt_offset ?? 0;
        const walls = [];
        const b = 1.0 / (2 * Math.sqrt(2.0));
        const h = -2.5 - b;
        const len = Math.sqrt(2 * 5 * 5);
        let w = Shapes.makeWall(len, 1, Shapes.TOP_EDGE, Walls.en.WALL_BOTTOM + '_' + PileConfig.en.LEFT + 1, Walls.i18n.WALL_BOTTOM + '_' + PileConfig.i18n.LEFT + 1);
        const w1 = w;
        w.setPosition(new Vector(-7.5 - b, h + offset), -Math.PI / 4);
        walls.push(w);
        w = Shapes.makeWall(len, 1, Shapes.TOP_EDGE, Walls.en.WALL_BOTTOM + '_' + PileConfig.en.LEFT + 2, Walls.i18n.WALL_BOTTOM + '_' + PileConfig.i18n.LEFT + 2);
        w.setPosition(new Vector(-2.5 + b, h + offset), Math.PI / 4);
        walls.push(w);
        w = Shapes.makeWall(len, 1, Shapes.TOP_EDGE, Walls.en.WALL_BOTTOM + '_' + PileConfig.en.RIGHT + 1, Walls.i18n.WALL_BOTTOM + '_' + PileConfig.i18n.RIGHT + 1);
        w.setPosition(new Vector(2.5 - b, h + offset), -Math.PI / 4);
        walls.push(w);
        w = Shapes.makeWall(len, 1, Shapes.TOP_EDGE, Walls.en.WALL_BOTTOM + '_' + PileConfig.en.RIGHT + 2, Walls.i18n.WALL_BOTTOM + '_' + PileConfig.i18n.RIGHT + 2);
        w.setPosition(new Vector(7.5 + b, h + offset), Math.PI / 4);
        walls.push(w);
        w = Shapes.makeWall(1, 15, Shapes.LEFT_EDGE, Walls.en.WALL_RIGHT, Walls.i18n.WALL_RIGHT);
        w.setPosition(new Vector(10.5, 7.5 + offset), 0);
        walls.push(w);
        w = Shapes.makeWall(1, 15, Shapes.RIGHT_EDGE, Walls.en.WALL_LEFT, Walls.i18n.WALL_LEFT);
        w.setPosition(new Vector(-10.5, 7.5 + offset), 0);
        walls.push(w);
        walls.forEach(p => {
            p.setMass(Infinity);
            sim.addBody(p);
            p.addNonCollide(walls);
        });
        return w1.bodyToWorld(new Vector(len / 2, 0.5)).getY();
    }
    ;
    static makeUniformBlocks(sim, rect, circular, size, buffer, limit) {
        let row = 0;
        let col = 0;
        let index = 0;
        while (true) {
            const y = rect.getBottom() + size + buffer + row * 2 * size
                + row * buffer;
            if (y + size + buffer > rect.getTop()) {
                break;
            }
            while (true) {
                const x = buffer + size + rect.getLeft() + col * 2 * size
                    + col * buffer;
                if (x + size + buffer > rect.getRight()) {
                    break;
                }
                let p;
                const id = index++;
                if (circular) {
                    p = Shapes.makeBall(size);
                }
                else {
                    p = Shapes.makeBlock(2 * size, 2 * size);
                }
                p.setPosition(new Vector(x, y));
                p.setMass(1);
                sim.addBody(p);
                if (index > limit) {
                    return;
                }
                col++;
            }
            col = 0;
            row++;
        }
    }
    ;
    static makeRandomBlocks(sim, n, x, y, random, rightAngle) {
        rightAngle = rightAngle !== undefined ? rightAngle : true;
        const bods = [];
        for (let i = 0; i < n; i++) {
            const width = 0.2 + random.nextFloat();
            const height = 0.2 + random.nextFloat();
            const angle = Math.PI * random.nextFloat();
            const unusedColor = PileConfig.getRandomColor(random);
            let p;
            if (rightAngle) {
                p = Shapes.makeBlock(width, height);
            }
            else {
                p = Shapes.makeRandomPolygon(4, Math.sqrt(width * width + height * height) / 2);
            }
            let cmx = width * random.nextFloat() / 4;
            let cmy = height * random.nextFloat() / 4;
            const xmin = 0.9 * width / 2.0;
            const ymin = 0.9 * height / 2.0;
            if (cmx < -xmin) {
                cmx = -xmin;
            }
            else if (cmx > xmin) {
                cmx = xmin;
            }
            if (cmy < -ymin) {
                cmy = -ymin;
            }
            else if (cmy > ymin) {
                cmy = ymin;
            }
            p.setCenterOfMass(new Vector(cmx, cmy));
            p.setPosition(new Vector(x, y), angle);
            const left = p.getLeftWorld();
            p.setPosition(new Vector(x + (x - left), y), angle);
            x = p.getRightWorld() + 0.05;
            p.setMass(p.getWidth() * p.getHeight());
            sim.addBody(p);
            bods.push(p);
        }
        return bods;
    }
    ;
    static makeConnectedBlocks(sim, x, y, angle) {
        const p1 = Shapes.makeBlock(1.0, 1.0);
        p1.setMass(0.6);
        p1.setPosition(new Vector(x, y), angle);
        sim.addBody(p1);
        const p2 = Shapes.makeBlock(0.9, 1.1);
        p2.setMass(0.6);
        p2.setPosition(new Vector(x, y), angle);
        sim.addBody(p2);
        JointUtil.attachRigidBody(sim, p1, new Vector(0, -0.4), p2, new Vector(-0.4, 0), 0);
        JointUtil.attachRigidBody(sim, p1, new Vector(0.4, 0), p2, new Vector(0, 0.4), 0);
        sim.alignConnectors();
        return [p1, p2];
    }
    ;
    static getRandomColor(random) {
        random = random || PileConfig.randomColor_;
        const colors = new Array(3);
        let nearWhite = true;
        for (let i = 0; i < 3; i++) {
            const c = Math.min(1, Math.max(0, 0.1 + 1.5 * random.nextFloat()));
            if (c < 0.9) {
                nearWhite = false;
            }
            colors[i] = c;
        }
        if (nearWhite) {
            return PileConfig.getRandomColor(random);
        }
        else {
            return Util.colorString6(colors[0], colors[1], colors[2]);
        }
    }
    ;
}
PileConfig.randomColor_ = new RandomLCG(901745);
PileConfig.en = {
    NUM_BLOCKS: 'number of blocks',
    ADD_BLOCK: 'add block',
    REBUILD: 'rebuild',
    TWO_PILES: 'two piles',
    CONNECTED_BLOCKS: 'connected blocks',
    ENDLESS_LOOP: 'endless loop',
    LOOP_TIME: 'loop time',
    RANDOM_SEED: 'random seed',
    WALL: 'wall',
    LEFT: 'left',
    RIGHT: 'right',
    SQUARE_BLOCKS: 'square blocks'
};
PileConfig.de_strings = {
    NUM_BLOCKS: 'Blöckezahl',
    ADD_BLOCK: 'Block hinzu fügen',
    REBUILD: 'wieder aufbauen',
    TWO_PILES: 'zwei Haufen',
    CONNECTED_BLOCKS: 'zusammenhängende Blöcke',
    ENDLESS_LOOP: 'endliche Schleife',
    LOOP_TIME: 'Schleifezeit',
    RANDOM_SEED: 'Zufallskern',
    WALL: 'Wand',
    LEFT: 'links',
    RIGHT: 'rechts',
    SQUARE_BLOCKS: 'rechteckig Blöcke'
};
PileConfig.i18n = Util.LOCALE === 'de' ? PileConfig.de_strings : PileConfig.en;
Util.defineGlobal('sims$engine2D$PileConfig', PileConfig);
