import { AbstractSubject } from "../../lab/util/AbstractSubject.js";
import { AutoScale } from "../../lab/graph/AutoScale.js";
import { ChoiceControl } from "../../lab/controls/ChoiceControl.js";
import { CheckBoxControl } from "../../lab/controls/CheckBoxControl.js";
import { ButtonControl } from "../../lab/controls/ButtonControl.js";
import { NumericControl } from "../../lab/controls/NumericControl.js";
import { CommonControls } from "../../sims/common/CommonControls.js";
import { DisplayAxes } from "../../lab/graph/DisplayAxes.js";
import { DisplayClock } from "../../lab/view/DisplayClock.js";
import { DisplayGraph } from '../../lab/graph/DisplayGraph.js';
import { DisplayLine } from "../../lab/view/DisplayLine.js";
import { DisplayArc } from "../../lab/view/DisplayArc.js";
import { DisplayPath } from "../../lab/view/DisplayPath.js";
import { DisplayShape } from "../../lab/view/DisplayShape.js";
import { DisplaySpring } from "../../lab/view/DisplaySpring.js";
import { DisplayText } from "../../lab/view/DisplayText.js";
import { DoubleRect } from "../../lab/util/DoubleRect.js";
import { EnergyBarGraph } from "../../lab/graph/EnergyBarGraph.js";
import { GraphLine } from '../../lab/graph/GraphLine.js';
import { LabCanvas } from "../../lab/view/LabCanvas.js";
import { NumericalPath } from "../../lab/model/NumericalPath.js";
import { SimpleAdvance } from "../../lab/model/SimpleAdvance.js";
import { SimController } from "../../lab/app/SimController.js";
import { SimRunner } from "../../lab/app/SimRunner.js";
import { SimView } from "../../lab/view/SimView.js";
import { Spring } from "../../lab/model/Spring.js";
import { Terminal } from "../../lab/util/Terminal.js";
import { Timer } from "../../lab/util/Timer.js";
import { Util } from "../../lab/util/Util.js";
import { ModifiedEuler } from "../../lab/model/ModifiedEuler.js";
import { PendulumSim } from "../pendulum/PendulumSim.js";
import { SingleSpringSim } from "../springs/SingleSpringSim.js";
import { RollerSingleSim } from "../roller/RollerSingleSim.js";
import { CirclePath } from "../roller/CirclePath.js";
import { CustomPath } from "../roller/CustomPath.js";
import { FlatPath } from "../roller/FlatPath.js";
import { OvalPath } from "../roller/OvalPath.js";
import { RigidBodyObserver } from "../engine2D/RigidBodyObserver.js";
import { MoleculeSim } from "../springs/MoleculeSim.js";
import { SpringNonLinear1 } from '../springs/SpringNonLinear1.js';
import { SpringNonLinear2 } from '../springs/SpringNonLinear2.js';
export class BlankSlateApp extends AbstractSubject {
    constructor(elem_ids) {
        Util.setErrorHandler();
        super('APP');
        this.debug_layout = true;
        this.z = {};
        Util.setImagesDir(elem_ids.images_dir);
        this.simDiv =
            Util.getElementById(elem_ids.sim_canvas);
        this.simDiv.style.position = 'relative';
        const canvas = document.createElement('canvas');
        this.simCanvas = new LabCanvas(canvas, 'canvas1');
        this.simCanvas.setSize(800, 800);
        this.simDiv.appendChild(this.simCanvas.getCanvas());
        this.controlsDiv =
            Util.getElementById(elem_ids.sim_controls);
        this.controlsDiv.style.marginLeft = '10px';
        if (this.debug_layout) {
            this.controlsDiv.style.border = 'dashed 1px green';
        }
        const term_input = Util.getElementById(elem_ids.term_input);
        const term_output = Util.getElementById(elem_ids.term_output);
        this.terminal = new Terminal(term_input, term_output);
        Terminal.stdRegex(this.terminal);
        this.simRect = new DoubleRect(-6, -6, 6, 6);
        this.simView = new SimView('simView', this.simRect);
        this.simCanvas.addView(this.simView);
        this.displayList = this.simView.getDisplayList();
        this.simCanvas.paint();
        this.simCtrl = new SimController(this.simCanvas);
        this.timer = new Timer();
        const callback = () => this.simCanvas.paint();
        this.timer.setCallBack(callback);
        this.timer.startFiring();
    }
    ;
    toString() {
        return this.toStringShort().slice(0, -1)
            + ', simCanvas: ' + this.simCanvas.toStringShort()
            + ', simRect: ' + this.simRect.toString()
            + ', simView: ' + this.simView.toStringShort()
            + ', displayList: ' + this.displayList.toStringShort()
            + ', timer: ' + this.timer.toString()
            + ', simCtrl: ' + this.simCtrl.toStringShort()
            + ', terminal: ' + this.terminal.toString()
            + super.toString();
    }
    ;
    getClassName() {
        return 'BlankSlateApp';
    }
    ;
    defineNames(myName) {
        this.terminal.addRegex('z', myName + '.', false, true);
        this.terminal.addRegex('simCanvas|terminal|axes|simCtrl|simView|displayList'
            + '|timer|controlsDiv|simDiv', myName + '.');
        this.terminal.addRegex('PendulumSim', 'sims$$pendulum$$', false);
        this.terminal.addRegex('MoleculeSim|SingleSpringSim|SpringNonLinear1'
            + '|SpringNonLinear2', 'sims$$springs$$', false);
        this.terminal.addRegex('RigidBodyObserver', 'sims$$engine2D$$', false);
        this.terminal.addRegex('RollerSingleSim|CirclePath|CustomPath|FlatPath|OvalPath', 'sims$$roller$$', false);
        this.terminal.addRegex('CommonControls', 'sims$$common$$', false);
    }
    ;
    eval(script, output = true) {
        return this.terminal.eval(script, output);
    }
    ;
    setup() {
        this.terminal.parseURL();
    }
    ;
    start() {
    }
    ;
    static loadClass() {
        var f = AutoScale.toString;
        f = ButtonControl.toString;
        f = CheckBoxControl.toString;
        f = ChoiceControl.toString;
        f = DisplayArc.toString;
        f = DisplayAxes.toString;
        f = DisplayClock.toString;
        f = DisplayGraph.toString;
        f = DisplayLine.toString;
        f = DisplayPath.toString;
        f = DisplayShape.toString;
        f = DisplaySpring.toString;
        f = EnergyBarGraph.toString;
        f = GraphLine.toString;
        f = ModifiedEuler.toString;
        f = NumericalPath.toString;
        f = NumericControl.toString;
        f = PendulumSim.toString;
        f = RollerSingleSim.toString;
        f = SimController.toString;
        f = SimpleAdvance.toString;
        f = SimRunner.toString;
        f = SingleSpringSim.toString;
        f = Spring.toString;
        f = SpringNonLinear1.toString;
        f = SpringNonLinear2.toString;
        f = CirclePath.toString;
        f = CustomPath.toString;
        f = FlatPath.toString;
        f = OvalPath.toString;
        f = DisplayText.toString;
        f = RigidBodyObserver.toString;
        f = MoleculeSim.toString;
        var p = CommonControls.makePanZoomControls;
    }
    ;
    addControl(control) {
        const element = control.getElement();
        element.style.display = 'block';
        this.controlsDiv.appendChild(element);
    }
    ;
}
Util.defineGlobal('sims$experimental$BlankSlateApp', BlankSlateApp);
