import { DisplayAxes } from '../../lab/graph/DisplayAxes.js';
import { DisplayGraph } from '../../lab/graph/DisplayGraph.js';
import { DoubleRect } from '../../lab/util/DoubleRect.js';
import { GenericObserver } from '../../lab/util/Observe.js';
import { GraphLine } from '../../lab/graph/GraphLine.js';
import { LabCanvas } from '../../lab/view/LabCanvas.js';
import { SimView } from '../../lab/view/SimView.js';
import { SimController } from '../../lab/app/SimController.js';
import { Terminal } from '../../lab/util/Terminal.js';
import { Util } from '../../lab/util/Util.js';
import { VarsList } from '../../lab/model/VarsList.js';
export class GraphCalcApp {
    constructor(elem_ids) {
        this.x = 0;
        Util.setErrorHandler();
        Util.setImagesDir(elem_ids['images_dir']);
        const div = Util.getElementById(elem_ids.graph_div);
        const canvas = document.createElement('canvas');
        this.simCanvas = new LabCanvas(canvas, 'canvas1');
        this.simCanvas.setSize(800, 800);
        div.appendChild(this.simCanvas.getCanvas());
        const term_input = Util.maybeElementById(elem_ids.term_input);
        const term_output = Util.maybeElementById(elem_ids.term_output);
        this.terminal = new Terminal(term_input, term_output);
        Terminal.stdRegex(this.terminal);
        this.vars = new VarsList(['x', 'y'], ['x', 'y']);
        this.simRect = new DoubleRect(-10, -10, 10, 10);
        this.simView = new SimView('graphView', this.simRect);
        this.simView.setHorizAlign("FULL");
        this.simView.setVerticalAlign("FULL");
        this.simCanvas.addView(this.simView);
        const screenrect = this.simView.getScreenRect();
        this.graphLine = new GraphLine('GRAPH_LINE', this.vars);
        this.graphLine.setXVariable(0);
        this.graphLine.setYVariable(1);
        this.graphLine.setHotSpotColor('');
        this.graph = new DisplayGraph(this.graphLine);
        this.graph.setScreenRect(screenrect);
        this.simView.getDisplayList().add(this.graph);
        this.axes = new DisplayAxes(this.simView.getCoordMap().screenToSimRect(screenrect));
        this.simView.getDisplayList().add(this.axes);
        new GenericObserver(this.simView, (evt) => {
            if (evt.nameEquals(SimView.SCREEN_RECT_CHANGED)) {
                this.graph.setScreenRect(this.simView.getScreenRect());
            }
            else if (evt.nameEquals(SimView.SIM_RECT_CHANGED)) {
                const screenrect = this.simView.getScreenRect();
                const simrect = this.simView.getCoordMap().screenToSimRect(screenrect);
                this.axes.setSimRect(simrect);
            }
        }, 'adjust when screen rect changes');
        this.simCtrl = new SimController(this.simCanvas);
        this.terminal.setAfterEval(() => {
            this.graph.reset();
            this.simCanvas.paint();
        });
    }
    ;
    plot(expr, range1, range2, numPts) {
        const r = this.simView.getSimRect();
        range1 = range1 ?? r.getLeft();
        range2 = range2 ?? r.getRight();
        numPts = numPts || 200;
        const incr = (range2 - range1) / numPts;
        if (incr <= 0) {
            this.terminal.println('ERROR calling plot: range2 must be greater than range1');
            return;
        }
        numPts += 1;
        this.vars.incrSequence(0);
        this.vars.incrSequence(1);
        this.x = range1;
        let y = NaN;
        for (let i = 0; i < numPts; i++) {
            y = this.terminal.eval(expr, false);
            if (typeof y === 'number' && isFinite(y)) {
                this.vars.setValue(0, this.x, true);
                this.vars.setValue(1, y, true);
                this.graphLine.memorize();
                this.x += incr;
            }
            else {
                this.terminal.println('ERROR in plot: result is not a number ' + expr);
                return;
            }
        }
        this.simCanvas.paint();
    }
    ;
    defineNames(myName) {
        this.terminal.addRegex('z', myName + '.', false, true);
        this.terminal.addRegex('simCanvas|vars|terminal|graph|graphLine|axes|plot'
            + '|simCtrl|simView|x', myName + '.');
    }
    ;
    eval(script, output = true) {
        return this.terminal.eval(script, output);
    }
    ;
    setup() {
        this.terminal.parseURL();
    }
    ;
    start() {
    }
    ;
}
Util.defineGlobal('sims$experimental$GraphCalcApp', GraphCalcApp);
