import { ShapeType } from '../../lab/model/PointMass.js';
import { Util } from '../../lab/util/Util.js';
export class DisplayRobotWheel {
    constructor(wheel, spokes = 8) {
        this.changed_ = true;
        if (wheel.getShape() != ShapeType.OVAL || wheel.getWidth() != wheel.getHeight()) {
            throw 'wheel must be a circle';
        }
        this.wheel_ = wheel;
        if (spokes < 0) {
            throw 'spokes must be positive ' + spokes;
        }
        this.spokes_ = spokes;
    }
    ;
    toString() {
        return this.toStringShort().slice(0, -1)
            + ', zIndex: ' + this.getZIndex()
            + '}';
    }
    ;
    toStringShort() {
        return 'DisplayRobotWheel{wheel_: ' +
            this.wheel_.toStringShort() + '}';
    }
    ;
    contains(p_world) {
        const p_body = this.wheel_.worldToBody(p_world);
        return this.wheel_.getBoundsBody().contains(p_body);
    }
    ;
    draw(context, map) {
        const r = this.wheel_.getWidth() / 2;
        context.save();
        const sim_to_screen = map.getAffineTransform();
        const sim_to_screen_units = 1 / map.getScaleX();
        const body_to_screen = sim_to_screen.concatenate(this.wheel_.bodyToWorldTransform());
        body_to_screen.setTransform(context);
        context.beginPath();
        if (typeof context.ellipse === 'function') {
            context.moveTo(r, 0);
            context.ellipse(0, 0, r, r, 0, 0, 2 * Math.PI, false);
        }
        else {
            context.arc(0, 0, r, 0, 2 * Math.PI, false);
            context.closePath();
        }
        context.fillStyle = 'lightGray';
        context.fill();
        context.lineWidth = map.screenToSimScaleX(1);
        context.strokeStyle = 'red';
        context.stroke();
        for (let i = 0; i < this.spokes_; i++) {
            context.beginPath();
            context.moveTo(0, 0);
            const a = i * 2 * Math.PI / this.spokes_;
            context.lineTo(r * Math.cos(a), r * Math.sin(a));
            context.strokeStyle = i == 0 ? 'red' : 'black';
            context.stroke();
        }
        context.restore();
    }
    ;
    getChanged() {
        if (this.wheel_.getChanged() || this.changed_) {
            this.changed_ = false;
            return true;
        }
        return false;
    }
    ;
    getMassObjects() {
        return [this.wheel_];
    }
    ;
    getPosition() {
        return this.wheel_.getPosition();
    }
    ;
    getSimObjects() {
        return [this.wheel_];
    }
    ;
    getZIndex() {
        if (this.zIndex_ !== undefined) {
            return this.zIndex_;
        }
        else {
            return 0;
        }
    }
    ;
    isDragable() {
        return false;
    }
    ;
    setDragable(_dragable) {
    }
    ;
    setPosition(_position) {
    }
    ;
    setZIndex(zIndex) {
        this.zIndex_ = zIndex;
        this.changed_ = true;
    }
    ;
}
Util.defineGlobal('sims$misc$DisplayRobotWheel', DisplayRobotWheel);
