import { AbstractApp } from '../common/AbstractApp.js';
import { DisplayPath } from '../../lab/view/DisplayPath.js';
import { DisplayShape } from '../../lab/view/DisplayShape.js';
import { DoubleRect } from '../../lab/util/DoubleRect.js';
import { DrawingStyle } from '../../lab/view/DrawingStyle.js';
import { LagrangeRollerSim } from './LagrangeRollerSim.js';
import { NumericControl } from '../../lab/controls/NumericControl.js';
import { SimpleAdvance } from '../../lab/model/SimpleAdvance.js';
import { Util } from '../../lab/util/Util.js';
export class LagrangeRollerApp extends AbstractApp {
    constructor(elem_ids) {
        Util.setErrorHandler();
        const simRect = new DoubleRect(-6, -6, 6, 6);
        const sim = new LagrangeRollerSim();
        const advance = new SimpleAdvance(sim);
        super(elem_ids, simRect, sim, advance, sim, sim);
        this.simRect = sim.getPath().getBoundsWorld().scale(1.2);
        this.simView.setSimRect(this.simRect);
        this.ball1 = new DisplayShape(this.simList.getPointMass('ball1'));
        this.ball1.setFillStyle('blue');
        this.displayList.add(this.ball1);
        this.displayPath = new DisplayPath();
        this.displayPath.setScreenRect(this.simView.getScreenRect());
        this.displayPath.setZIndex(-1);
        this.displayList.add(this.displayPath);
        this.displayPath.addPath(sim.getPath(), DrawingStyle.lineStyle('gray', 4));
        this.addPlaybackControls();
        let pn = sim.getParameterNumber(LagrangeRollerSim.en.GRAVITY);
        this.addControl(new NumericControl(pn));
        pn = sim.getParameterNumber(LagrangeRollerSim.en.MASS);
        this.addControl(new NumericControl(pn));
        this.addStandardControls();
        this.graph.line.setXVariable(0);
        this.graph.line.setYVariable(1);
        this.timeGraph.line1.setYVariable(0);
        this.timeGraph.line2.setYVariable(1);
        this.makeEasyScript();
        this.addURLScriptButton();
    }
    ;
    toString() {
        return this.toStringShort().slice(0, -1)
            + ', ball1: ' + this.ball1.toStringShort()
            + ', displayPath: ' + this.displayPath.toStringShort()
            + super.toString();
    }
    ;
    getClassName() {
        return 'LagrangeRollerApp';
    }
    ;
    defineNames(myName) {
        super.defineNames(myName);
        this.terminal.addRegex('ball1', myName + '.');
    }
    ;
}
Util.defineGlobal('sims$roller$LagrangeRollerApp', LagrangeRollerApp);
