import { DisplayPath } from '../../lab/view/DisplayPath.js';
import { DoubleRect } from '../../lab/util/DoubleRect.js';
import { GenericObserver } from '../../lab/util/Observe.js';
import { NumericalPath } from '../../lab/model/NumericalPath.js';
import { SimList } from '../../lab/model/SimList.js';
import { SimView } from '../../lab/view/SimView.js';
import { Util } from '../../lab/util/Util.js';
export class PathObserver {
    constructor(simList, simView, simRectSetter, opt_expand) {
        this.simView_ = simView;
        this.displayList_ = simView.getDisplayList();
        this.simList_ = simList;
        this.simRectSetter_ = simRectSetter;
        this.expansionFactor_ = opt_expand || 1.1;
        this.simList_.addObserver(this);
        this.memObjs_ = [];
        this.protoDisplayPath = new DisplayPath();
        this.protoDisplayPath.setZIndex(-10);
    }
    ;
    toString() {
        return 'PathObserver{'
            + 'simList_: ' + this.simList_.toStringShort()
            + ', simView_: ' + this.simView_.toStringShort()
            + ', expansionFactor: ' + Util.NF(this.expansionFactor_)
            + ', displayList_: ' + this.displayList_.toStringShort()
            + ', protoDisplayPath: ' + (this.protoDisplayPath ?
            this.protoDisplayPath.toStringShort() : 'undefined')
            + '}';
    }
    ;
    toStringShort() {
        return 'PathObserver{}';
    }
    ;
    addPath(np) {
        if (this.displayList_.find(np) != null) {
            return;
        }
        const displayPath = new DisplayPath(this.protoDisplayPath);
        displayPath.setScreenRect(this.simView_.getScreenRect());
        displayPath.addPath(np);
        this.displayList_.add(displayPath);
        if (this.simRectSetter_ != null) {
            let r = np.getBoundsWorld().scale(this.expansionFactor_);
            if (r.isEmpty()) {
                const unitRect = DoubleRect.makeCentered(r.getCenter(), 1, 1);
                r = r.union(unitRect);
            }
            this.simRectSetter_(r);
        }
        const obs = new GenericObserver(this.simView_, event => {
            if (event.getSubject() == this.simView_) {
                if (event.nameEquals(SimView.SCREEN_RECT_CHANGED)) {
                    displayPath.setScreenRect(this.simView_.getScreenRect());
                }
            }
        }, 'resize displayPath when screenRect changes');
        this.memObjs_.push({ simObj: np, obs: obs, dispPath: displayPath });
    }
    ;
    removePath(np) {
        const memObj = this.memObjs_.find(element => element.simObj == np);
        if (memObj !== undefined) {
            this.displayList_.remove(memObj.dispPath);
            memObj.obs.disconnect();
            memObj.dispPath.removePath(np);
            Util.remove(this.memObjs_, memObj);
        }
    }
    ;
    observe(event) {
        if (event.getSubject() == this.simList_) {
            const obj = event.getValue();
            if (obj instanceof NumericalPath) {
                if (event.nameEquals(SimList.OBJECT_ADDED)) {
                    this.addPath(obj);
                }
                else if (event.nameEquals(SimList.OBJECT_REMOVED)) {
                    this.removePath(obj);
                }
            }
        }
    }
    ;
}
Util.defineGlobal('sims$roller$PathObserver', PathObserver);
