import { AbstractApp } from '../common/AbstractApp.js';
import { ButtonControl } from '../../lab/controls/ButtonControl.js';
import { CommonControls } from '../common/CommonControls.js';
import { DisplayShape } from '../../lab/view/DisplayShape.js';
import { DisplaySpring } from '../../lab/view/DisplaySpring.js';
import { DoubleRect } from '../../lab/util/DoubleRect.js';
import { EnergyInfo } from '../../lab/model/EnergySystem.js';
import { NumericControl } from '../../lab/controls/NumericControl.js';
import { SimpleAdvance } from '../../lab/model/SimpleAdvance.js';
import { SliderControl } from '../../lab/controls/SliderControl.js';
import { Spring2DSim } from './Spring2DSim.js';
import { Util } from '../../lab/util/Util.js';
export class Spring2DApp extends AbstractApp {
    constructor(elem_ids) {
        Util.setErrorHandler();
        const simRect = new DoubleRect(-6, -6, 6, 6);
        const sim = new Spring2DSim();
        const advance = new SimpleAdvance(sim);
        super(elem_ids, simRect, sim, advance, sim, sim);
        this.layout.getSimCanvas().setBackground('black');
        this.layout.getSimCanvas().setAlpha(CommonControls.SHORT_TRAILS);
        this.anchor = new DisplayShape(this.simList.getPointMass('anchor'));
        this.anchor.setFillStyle('');
        this.anchor.setStrokeStyle('red');
        this.anchor.setThickness(4);
        this.displayList.add(this.anchor);
        this.bob = new DisplayShape(this.simList.getPointMass('bob'));
        this.bob.setFillStyle('blue');
        this.displayList.add(this.bob);
        this.spring = new DisplaySpring(this.simList.getSpring('spring'));
        this.spring.setWidth(0.3);
        this.displayList.add(this.spring);
        sim.saveInitialState();
        this.addPlaybackControls();
        let pn = sim.getParameterNumber(Spring2DSim.en.GRAVITY);
        this.addControl(new SliderControl(pn, 0, 20, false));
        pn = sim.getParameterNumber(Spring2DSim.en.MASS);
        this.addControl(new SliderControl(pn, 0.2, 20.2, true));
        pn = sim.getParameterNumber(Spring2DSim.en.DAMPING);
        this.addControl(new SliderControl(pn, 0, 1, false));
        pn = sim.getParameterNumber(Spring2DSim.en.SPRING_LENGTH);
        this.addControl(new SliderControl(pn, 0.1, 10.1, true));
        pn = sim.getParameterNumber(Spring2DSim.en.SPRING_STIFFNESS);
        this.addControl(new SliderControl(pn, 0.1, 100.1, true));
        pn = sim.getParameterNumber(EnergyInfo.en.PE_OFFSET);
        this.addControl(new NumericControl(pn));
        this.addStandardControls();
        const bc = new ButtonControl(Spring2DSim.i18n.REST_STATE, () => sim.restState());
        this.addControl(bc);
        this.makeEasyScript();
        this.addURLScriptButton();
    }
    ;
    toString() {
        return this.toStringShort().slice(0, -1)
            + ', bob: ' + this.bob.toStringShort()
            + ', spring: ' + this.bob.toStringShort()
            + ', anchor: ' + this.anchor.toStringShort()
            + super.toString();
    }
    ;
    getClassName() {
        return 'Spring2DApp';
    }
    ;
    defineNames(myName) {
        super.defineNames(myName);
        this.terminal.addRegex('anchor|bob|spring', myName + '.');
    }
    ;
}
Util.defineGlobal('sims$springs$Spring2DApp', Spring2DApp);
