import { Force } from '../../lab/model/Force.js';
import { Spring } from '../../lab/model/Spring.js';
import { Util } from '../../lab/util/Util.js';
import { Vector } from '../../lab/util/Vector.js';
export class SpringNonLinear1 extends Spring {
    constructor(name, body1, attach1_body, body2, attach2_body, restLength, stiffness) {
        super(name, body1, attach1_body, body2, attach2_body, restLength, stiffness, false);
        this.minPE_ = 0;
        this.calcMinPE();
    }
    ;
    getClassName() {
        return 'SpringNonLinear1';
    }
    ;
    calculateForces() {
        const point1 = this.getStartPoint();
        const point2 = this.getEndPoint();
        const body1 = this.getBody1();
        const body2 = this.getBody2();
        const v = point2.subtract(point1);
        const len = v.length();
        const sf = -this.getStiffness() * (6 * Math.pow(len, -1) - Math.pow(len / 2, -3));
        const fx = -sf * (v.getX() / len);
        const fy = -sf * (v.getY() / len);
        let f = new Vector(fx, fy, 0);
        if (this.getDamping() != 0) {
            const v1 = body1.getVelocity(this.getAttach1());
            const v2 = body2.getVelocity(this.getAttach2());
            const df = v1.subtract(v2).multiply(-this.getDamping());
            f = f.add(df);
        }
        return [new Force('spring', body1, point1, 1, f, 1),
            new Force('spring', body2, point2, 1, f.multiply(-1), 1)];
    }
    ;
    getPotentialEnergy() {
        const len = this.getLength();
        return this.potentialEnergy(len) - this.minPE_;
    }
    ;
    potentialEnergy(len) {
        const S = this.getStiffness();
        return S * (6 * Math.log(len) + 4 / (len * len));
    }
    ;
    minPELen() {
        return 2 / Math.sqrt(3);
    }
    ;
    calcMinPE() {
        this.minPE_ = this.potentialEnergy(this.minPELen());
    }
    ;
}
Util.defineGlobal('sims$springs$SpringNonLinear1', SpringNonLinear1);
