import { CollisionAdvance } from '../lab/model/CollisionAdvance.js';
import { ConcreteLine } from '../lab/model/ConcreteLine.js';
import { ContactSim } from '../lab/engine2D/ContactSim.js';
import { DampingLaw } from '../lab/model/DampingLaw.js';
import { GravityLaw } from '../lab/model/GravityLaw.js';
import { Joint } from '../lab/engine2D/Joint.js';
import { JointUtil } from '../lab/engine2D/JointUtil.js';
import { PointMass } from '../lab/model/PointMass.js';
import { RungeKutta } from '../lab/model/RungeKutta.js';
import { Shapes } from '../lab/engine2D/Shapes.js';
import { Vector } from '../lab/util/Vector.js';
import { schedule, setTestName } from './TestRig.js';
import { makeVars, setBodyVars, runTest, checkTightJoints } from './Engine2DTestRig.js';
const groupName = 'JointTest.';
;
export class JointTest {
    constructor() { throw ''; }
    ;
    static test() {
        schedule(JointTest.pendulum_1_joint_1);
        schedule(JointTest.pendulum_1_joint_2);
        schedule(JointTest.pendulum_2_joints_1);
        schedule(JointTest.pendulum_2_joints_offset_1);
        schedule(JointTest.two_blocks_2_dbl_joint_1);
    }
    ;
    static commonSetup1(sim, advance) {
        sim.addForceLaw(new DampingLaw(0, 0.15, sim.getSimList()));
        sim.setDistanceTol(0.01);
        sim.setVelocityTol(0.5);
        sim.setCollisionAccuracy(0.6);
        sim.setCollisionHandling("serial grouped lastpass");
        sim.setRandomSeed(99999);
        if (sim instanceof ContactSim) {
            sim.setExtraAccel("none");
        }
        advance.setJointSmallImpacts(true);
        advance.setTimeStep(0.025);
        advance.setDiffEqSolver(new RungeKutta(sim));
    }
    ;
    static buildJointTest(sim, testConfig, normalType) {
        switch (testConfig) {
            case 1:
            case 2:
            case 4:
            case 3:
                JointTest.makeBlockPendulum(sim, testConfig, normalType);
                break;
            case 5:
            case 6:
            case 7:
            case 8:
            case 9:
            case 10:
                JointTest.makeConnectedBlocks(sim, testConfig, normalType);
                break;
            default:
                throw '';
        }
    }
    ;
    static makeBlockPendulum(sim, testConfig, normalType) {
        Joint.nextJointNum = 0;
        const p1 = Shapes.makeBlock(1.0, 5.0, 'PENDULUM');
        p1.setDragPoints([new Vector(0.0, -2.0)]);
        p1.setPosition(new Vector(0.0, -2.0), 0.0);
        p1.setZeroEnergyLevel();
        sim.addBody(p1);
        const attach1 = new Vector(0.0, 2.0);
        const attach2 = new Vector(0.0, 0.0);
        const attach3 = new Vector(-0.4, -1.0);
        const normal0 = Vector.NORTH;
        const normal1 = Vector.EAST;
        switch (testConfig) {
            case 2:
                JointUtil.addSingleFixedJoint(sim, p1, attach1, normalType, normal1);
                break;
            case 4:
                JointUtil.addSingleFixedJoint(sim, p1, attach3, normalType, normal0);
            case 3:
                JointUtil.addSingleFixedJoint(sim, p1, attach2, normalType, normal1);
                break;
            default:
        }
        {
            JointUtil.addSingleFixedJoint(sim, p1, attach1, normalType, normal0);
            const pt1 = PointMass.makeCircle(0.2, 'ATTACH1');
            pt1.setMass(Infinity);
            pt1.setPosition(p1.bodyToWorld(attach1));
            sim.getSimList().add(pt1);
        }
        sim.setElasticity(0.8);
        sim.alignConnectors();
    }
    ;
    static makeConnectedBlocks(sim, testConfig, normalType) {
        Joint.nextJointNum = 0;
        const x = 0;
        const y = 0;
        const angle = 0;
        const zeroEnergyLevel = 0;
        const p1 = Shapes.makeBlock(1.0, 1.0, 'connect1');
        const p2 = Shapes.makeBlock(0.9, 1.1, 'connect2');
        p1.setMass(0.6);
        p2.setMass(0.6);
        p1.setPosition(new Vector(x, y), angle);
        p2.setPosition(new Vector(x, y), angle);
        sim.addBody(p1);
        sim.addBody(p2);
        p1.setZeroEnergyLevel(zeroEnergyLevel);
        p2.setZeroEnergyLevel(zeroEnergyLevel);
        const attach1 = new Vector(0, -0.4);
        const attach2 = new Vector(-0.4, 0);
        const attach3 = new Vector(0.4, 0);
        const attach4 = new Vector(0, 0.4);
        const attach5 = new Vector(0, 0);
        const attach6 = new Vector(-0.4, 0.4);
        const attach7 = new Vector(0.4, -0.4);
        const attach8 = new Vector(0.0, 0.0);
        const normal0 = Vector.NORTH;
        const normal1 = Vector.EAST;
        const normal2 = (new Vector(0.2, 0.8)).normalize();
        const normal6 = (new Vector(-0.5, 0.5)).normalize();
        const normal4 = Vector.EAST;
        const normal8 = (new Vector(0.6, -0.4)).normalize();
        switch (testConfig) {
            case 8:
                JointUtil.addSingleJoint(sim, p1, attach7, p2, attach8, normalType, normal8);
            case 7:
                JointUtil.addSingleJoint(sim, p1, attach5, p2, attach6, normalType, normal6);
            case 6:
                JointUtil.addSingleJoint(sim, p1, attach3, p2, attach4, normalType, normal4);
            case 5:
                JointUtil.addSingleJoint(sim, p1, attach1, p2, attach2, normalType, normal2);
                break;
            case 10:
                JointUtil.addSingleJoint(sim, p1, attach3, p2, attach4, normalType, normal0);
                JointUtil.addSingleJoint(sim, p1, attach3, p2, attach4, normalType, normal1);
            case 9:
                JointUtil.addSingleJoint(sim, p1, attach1, p2, attach2, normalType, normal0);
                JointUtil.addSingleJoint(sim, p1, attach1, p2, attach2, normalType, normal1);
                break;
            default: throw '';
        }
        sim.setElasticity(0.8);
        sim.alignConnectors();
    }
    ;
    static pendulum_1_joint_setup(sim, advance) {
        JointTest.commonSetup1(sim, advance);
        JointTest.buildJointTest(sim, 1, 1);
        const pendulum = sim.getBody('pendulum');
        pendulum.setAngle(Math.PI / 3);
        sim.alignConnectors();
        sim.addForceLaw(new GravityLaw(5.0, sim.getSimList()));
        const line = new ConcreteLine('joint-line', new Vector(0, 0), new Vector(4, 0));
        sim.getSimList().add(line);
    }
    ;
    static pendulum_1_joint_1() {
        setTestName(groupName + 'pendulum_1_joint_1');
        const sim = new ContactSim();
        const advance = new CollisionAdvance(sim);
        JointTest.pendulum_1_joint_setup(sim, advance);
        const vars = makeVars(6);
        setBodyVars(sim, vars, 0, 1.7320508, 0, -1.3687651, -1.3515045, -0.8170398, 0.9268013);
        runTest(sim, advance, 15.0, vars, 0.00001, 0.0, 0.00001, -1);
        checkTightJoints(sim, 0.000001);
    }
    ;
    static pendulum_1_joint_2() {
        setTestName(groupName + 'pendulum_1_joint_2');
        const sim = new ContactSim();
        const advance = new CollisionAdvance(sim);
        JointTest.pendulum_1_joint_setup(sim, advance);
        advance.setJointSmallImpacts(false);
        sim.setExtraAccel("velocity_and_distance_joints");
        const vars = makeVars(6);
        setBodyVars(sim, vars, 0, 1.7320508, 0, -1.3680947, -1.3501097, -0.8175124, 0.9262573);
        runTest(sim, advance, 15.0, vars, 0.00001, 0.0, 0.001, -1);
        checkTightJoints(sim, 0.00012);
    }
    ;
    static pendulum_2_joints_setup(sim, advance) {
        JointTest.commonSetup1(sim, advance);
        JointTest.buildJointTest(sim, 2, 1);
        const pendulum = sim.getBody('pendulum');
        pendulum.setAngle(Math.PI / 3);
        sim.alignConnectors();
        sim.addForceLaw(new GravityLaw(5.0, sim.getSimList()));
    }
    ;
    static pendulum_2_joints_1() {
        setTestName(groupName + 'pendulum_2_joints_1');
        const sim = new ContactSim();
        const advance = new CollisionAdvance(sim);
        JointTest.pendulum_2_joints_setup(sim, advance);
        const vars = makeVars(6);
        setBodyVars(sim, vars, 0, -1.7297436, 0.0807141, -1.0039856, -0.1390604, -1.0448949, 0.0803936);
        runTest(sim, advance, 8.0, vars, 0.00001, 0.0, 0.00001, -1);
        checkTightJoints(sim, 0.000001);
    }
    ;
    static pendulum_2_joints_offset_setup(sim, advance) {
        JointTest.commonSetup1(sim, advance);
        JointTest.buildJointTest(sim, 3, 1);
        const pendulum = sim.getBody('pendulum');
        pendulum.setAngle(Math.PI / 3);
        sim.alignConnectors();
        sim.addForceLaw(new GravityLaw(5.0, sim.getSimList()));
        {
            const joint = sim.getSimList().get('JOINT1');
            const line = new ConcreteLine('joint1_line', joint.getPosition1().add(new Vector(-4, 0)), joint.getPosition1().add(new Vector(4, 0)));
            sim.getSimList().add(line);
        }
        {
            const joint = sim.getSimList().get('JOINT0');
            const x = pendulum.getPosition().getX();
            const line = new ConcreteLine('joint0_line', joint.getPosition1().add(new Vector(x, -4)), joint.getPosition1().add(new Vector(x, 4)));
            sim.getSimList().add(line);
        }
    }
    ;
    static pendulum_2_joints_offset_1() {
        setTestName(groupName + 'pendulum_2_joints_offset_1');
        const sim = new ContactSim();
        const advance = new CollisionAdvance(sim);
        JointTest.pendulum_2_joints_offset_setup(sim, advance);
        const vars = makeVars(6);
        setBodyVars(sim, vars, 0, 1.7320508, 0, -1.1100682, 0.7857346, 0.9823923, 0.4722942);
        runTest(sim, advance, 8.0, vars, 0.00001, 0.0, 0.00001, -1);
    }
    ;
    static pendulum_3_joints_offset_setup(sim, advance) {
        JointTest.commonSetup1(sim, advance);
        JointTest.buildJointTest(sim, 4, 1);
        const pendulum = sim.getBody('pendulum');
        pendulum.setAngle(Math.PI / 3);
        sim.alignConnectors();
        sim.addForceLaw(new GravityLaw(5.0, sim.getSimList()));
    }
    ;
    static two_blocks_1_joint_setup(sim, advance) {
        JointTest.commonSetup1(sim, advance);
        JointTest.buildJointTest(sim, 5, 0);
    }
    ;
    static two_blocks_2_joints_setup(sim, advance) {
        JointTest.commonSetup1(sim, advance);
        JointTest.buildJointTest(sim, 6, 0);
    }
    ;
    static two_blocks_3_joints_setup(sim, advance) {
        JointTest.commonSetup1(sim, advance);
        JointTest.buildJointTest(sim, 7, 0);
    }
    ;
    static two_blocks_4_joints_setup(sim, advance) {
        JointTest.commonSetup1(sim, advance);
        JointTest.buildJointTest(sim, 8, 0);
        const body1 = sim.getBody('connect1');
        body1.setVelocity(new Vector(0, 0), 6);
        const collisions = [];
        sim.findCollisions(collisions, sim.getVarsList().getValues(), 0);
        sim.handleCollisions(collisions);
    }
    ;
    static two_blocks_1_dbl_joint_setup(sim, advance) {
        JointTest.commonSetup1(sim, advance);
        JointTest.buildJointTest(sim, 9, 0);
    }
    ;
    static two_blocks_2_dbl_joint_setup(sim, advance) {
        JointTest.commonSetup1(sim, advance);
        JointTest.buildJointTest(sim, 10, 0);
        const body1 = sim.getBody('connect1');
        body1.setVelocity(new Vector(0, 0), 6);
        sim.setElasticity(0.8);
        const collisions = [];
        sim.findCollisions(collisions, sim.getVarsList().getValues(), 0);
        sim.handleCollisions(collisions);
    }
    ;
    static two_blocks_2_dbl_joint_1() {
        setTestName(groupName + 'two_blocks_2_dbl_joint_1');
        const sim = new ContactSim();
        const advance = new CollisionAdvance(sim);
        JointTest.two_blocks_2_dbl_joint_setup(sim, advance);
        const vars = makeVars(6 * 2);
        setBodyVars(sim, vars, 0, 0.4674208, 0.1861004, -0.2921196, 0.5402443, 16.1616176, 2.0202021);
        setBodyVars(sim, vars, 1, -0.0674208, -0.1861004, -0.1078804, -0.5402443, 16.1616176, 2.0202021);
        runTest(sim, advance, 8.0, vars, 0.00001, 0.0, 0.00001, -1);
        checkTightJoints(sim, 0.000001);
    }
    ;
}
