import { CollisionAdvance } from '../lab/model/CollisionAdvance.js';
import { ContactSim } from '../lab/engine2D/ContactSim.js';
import { DampingLaw } from '../lab/model/DampingLaw.js';
import { DoubleRect } from '../lab/util/DoubleRect.js';
import { GravityLaw } from '../lab/model/GravityLaw.js';
import { ModifiedEuler } from '../lab/model/ModifiedEuler.js';
import { PileConfig } from '../sims/engine2D/PileConfig.js';
import { RandomLCG } from '../lab/util/Random.js';
import { Shapes } from '../lab/engine2D/Shapes.js';
import { Util } from '../lab/util/Util.js';
import { Vector } from '../lab/util/Vector.js';
import { schedule, setTestName, reportTestResults, myPrintln, perfExpected, perfResult, getPerfLimit, testName } from './TestRig.js';
import { makeVars, setBodyVars, runTest, checkTightJoints } from './Engine2DTestRig.js';
const groupName = 'PileTest.';
export class PileTest {
    constructor() { throw ''; }
    ;
    static test() {
        schedule(PileTest.pile_config_1_test);
        schedule(PileTest.near_stable_connected_blocks_pile_test);
        schedule(PileTest.stable_connected_blocks_pile_test);
        schedule(PileTest.pile_10_random_blocks);
        schedule(PileTest.connected_blocks_pile_test);
    }
    ;
    static testPerformance() {
        schedule(PileTest.pile_10_perf);
    }
    ;
    static connected_blocks_pile_setup(sim, advance) {
        sim.setSimRect(new DoubleRect(-3.5, -9.2, 3.5, -1));
        sim.addForceLaw(new DampingLaw(0.05, 0.15, sim.getSimList()));
        sim.setShowForces(false);
        sim.setCollisionHandling("serial grouped lastpass");
        sim.setCollisionAccuracy(0.6);
        sim.setRandomSeed(12);
        sim.setExtraAccel("velocity_and_distance");
        advance.setDiffEqSolver(new ModifiedEuler(sim));
        advance.setJointSmallImpacts(true);
        advance.setTimeStep(0.025);
        const zel = PileConfig.makeVPit(sim);
        const buildRNG = new RandomLCG(4);
        PileConfig.makeRandomBlocks(sim, 5, -6, -2, buildRNG);
        PileConfig.makeConnectedBlocks(sim, 6, 7, Math.PI / 8 + 0.05);
        sim.setElasticity(0.8);
        const gravityLaw = new GravityLaw(10.0);
        sim.addForceLaw(gravityLaw);
        gravityLaw.addBodies(sim.getBodies());
        gravityLaw.setZeroEnergyLevel(zel);
    }
    ;
    static connected_blocks_pile_test() {
        setTestName(groupName + 'connected_blocks_pile_test');
        const sim = new ContactSim();
        const advance = new CollisionAdvance(sim);
        PileTest.connected_blocks_pile_setup(sim, advance);
        runTest(sim, advance, 10.0, null, NaN, NaN, 0, -1, -1);
        runTest(sim, advance, 15.0, null, NaN, 0.0, 0.0001, 0);
        checkTightJoints(sim, 0.025);
    }
    ;
    static stable_connected_blocks_pile_setup(sim, advance) {
        sim.setSimRect(new DoubleRect(-3.5, -9.2, 3.5, -1));
        sim.addForceLaw(new DampingLaw(0.05, 0.15, sim.getSimList()));
        sim.setShowForces(false);
        sim.setCollisionHandling("serial grouped lastpass");
        sim.setCollisionAccuracy(0.6);
        sim.setRandomSeed(0);
        sim.setExtraAccel("velocity_and_distance");
        advance.setDiffEqSolver(new ModifiedEuler(sim));
        advance.setJointSmallImpacts(true);
        advance.setTimeStep(0.025);
        const zel = PileConfig.makeVPit(sim);
        const buildRNG = new RandomLCG(4);
        PileConfig.makeRandomBlocks(sim, 5, -6, -2, buildRNG);
        PileConfig.makeConnectedBlocks(sim, 6, 7, Math.PI / 8 + 0.05);
        sim.setElasticity(0.8);
        const vars = sim.getVarsList();
        vars.setValue(28, -1.307317532081011);
        vars.setValue(29, 3.4568078175222356e-14);
        vars.setValue(30, -7.679563403384621);
        vars.setValue(31, -4.2635556397879164e-14);
        vars.setValue(32, 3.9269908169872414);
        vars.setValue(33, -1.3993648320570288e-15);
        vars.setValue(34, -0.6338649363183791);
        vars.setValue(35, 2.230623502423369e-14);
        vars.setValue(36, -8.36840450674311);
        vars.setValue(37, -9.196703672775336e-15);
        vars.setValue(38, 3.926990816987241);
        vars.setValue(39, -1.8602527688658332e-14);
        vars.setValue(40, -0.863509453866681);
        vars.setValue(41, 2.5327666670337238e-14);
        vars.setValue(42, -7.443359658566195);
        vars.setValue(43, -3.609941383157646e-14);
        vars.setValue(44, 16.493361431346415);
        vars.setValue(45, -3.834636183127082e-14);
        vars.setValue(46, -0.2794896421175775);
        vars.setValue(47, 2.372714000478332e-14);
        vars.setValue(48, -8.88957479858616);
        vars.setValue(49, -5.4309467895987256e-15);
        vars.setValue(50, 21.205750411731103);
        vars.setValue(51, -7.082394166687482e-14);
        vars.setValue(52, 0.1624361210614297);
        vars.setValue(53, 6.727726003835136e-13);
        vars.setValue(54, -8.875842608812114);
        vars.setValue(55, 1.6018768653977866e-12);
        vars.setValue(56, -0.7853981633975663);
        vars.setValue(57, -1.3844587651630329e-12);
        vars.setValue(58, 0.7732428782686775);
        vars.setValue(59, -4.652887853860082e-11);
        vars.setValue(60, -7.805472491546362);
        vars.setValue(61, -4.652474225205978e-11);
        vars.setValue(62, 3.926990816987241);
        vars.setValue(63, 3.6456541219008105e-15);
        vars.setValue(64, 0.19928633339146276);
        vars.setValue(65, 4.859438051687864e-11);
        vars.setValue(66, -7.791391682401767);
        vars.setValue(67, -4.718455523774639e-11);
        vars.setValue(68, 3.9269908169437704);
        vars.setValue(69, -4.409839525917456e-12);
        sim.modifyObjects();
        sim.saveInitialState();
        sim.setRandomSeed(1);
        const gravityLaw = new GravityLaw(10.0);
        sim.addForceLaw(gravityLaw);
        gravityLaw.addBodies(sim.getBodies());
        gravityLaw.setZeroEnergyLevel(zel);
    }
    ;
    static stable_connected_blocks_pile_test() {
        setTestName(groupName + 'stable_connected_blocks_pile_test');
        const sim = new ContactSim();
        const advance = new CollisionAdvance(sim);
        PileTest.stable_connected_blocks_pile_setup(sim, advance);
        const vars = makeVars(11 * 6);
        setBodyVars(sim, vars, 0, -5, 0, -5, 0, -0.7853982, 0);
        setBodyVars(sim, vars, 1, 5, 0, -5, 0, 0.7853982, 0);
        setBodyVars(sim, vars, 2, 10.5, 0, 7.5, 0, 0, 0);
        setBodyVars(sim, vars, 3, -10.5, 0, 7.5, 0, 0, 0);
        setBodyVars(sim, vars, 4, -1.3073175, -0, -7.6795634, 0, 3.9269908, -0);
        setBodyVars(sim, vars, 5, -0.6338649, -0, -8.3684045, 0, 3.9269908, -0);
        setBodyVars(sim, vars, 6, -0.8635095, -0, -7.4433597, 0, 16.4933614, -0);
        setBodyVars(sim, vars, 7, -0.2794896, 0, -8.8895748, 0, 21.2057504, -0);
        setBodyVars(sim, vars, 8, 0.1624361, 0, -8.8758426, 0, -0.7853982, -0);
        setBodyVars(sim, vars, 9, 0.7732429, -0, -7.8054725, 0, 3.9269908, -0);
        setBodyVars(sim, vars, 10, 0.1992863, -0, -7.7913917, 0, 3.9269908, 0);
        runTest(sim, advance, 10, vars, 0.00001, 0.0, 0.00001, 0);
        checkTightJoints(sim, 0.025);
    }
    ;
    static near_stable_connected_blocks_pile_setup(sim, advance) {
        sim.setSimRect(new DoubleRect(-3.5, -9.2, 3.5, -1));
        sim.addForceLaw(new DampingLaw(0.05, 0.15, sim.getSimList()));
        sim.setShowForces(false);
        sim.setCollisionHandling("serial grouped lastpass");
        sim.setCollisionAccuracy(0.6);
        sim.setRandomSeed(0);
        sim.setExtraAccel("velocity_and_distance");
        advance.setDiffEqSolver(new ModifiedEuler(sim));
        advance.setJointSmallImpacts(true);
        advance.setTimeStep(0.025);
        const zel = PileConfig.makeVPit(sim);
        const buildRNG = new RandomLCG(4);
        PileConfig.makeRandomBlocks(sim, 5, -6, -2, buildRNG);
        PileConfig.makeConnectedBlocks(sim, 6, 7, Math.PI / 8 + 0.05);
        sim.setElasticity(0.8);
        const vars = sim.getVarsList();
        vars.setValue(28, -1.3074166349097716);
        vars.setValue(29, -0.010840525228905605);
        vars.setValue(30, -7.6748558792433785);
        vars.setValue(31, 0.03162290166164498);
        vars.setValue(32, 3.9269569334324403);
        vars.setValue(33, 0.004326577793446015);
        vars.setValue(34, -0.6295172375406212);
        vars.setValue(35, -0.12156323274219026);
        vars.setValue(36, -8.363693937673977);
        vars.setValue(37, -0.11589631970547537);
        vars.setValue(38, 3.9243199960788253);
        vars.setValue(39, -0.07310382168395342);
        vars.setValue(40, 0.31893418138882307);
        vars.setValue(41, -0.6319951813598994);
        vars.setValue(42, -6.8097160566956365);
        vars.setValue(43, 2.1006107619247305);
        vars.setValue(44, 15.6167669356595);
        vars.setValue(45, 6.9807796520379455);
        vars.setValue(46, -0.2794020111788371);
        vars.setValue(47, 0.005559355507812818);
        vars.setValue(48, -8.889415521500014);
        vars.setValue(49, 0.012931603383049635);
        vars.setValue(50, 21.205702274040966);
        vars.setValue(51, -0.02672084560754362);
        vars.setValue(52, 0.16264515275973984);
        vars.setValue(53, 0.03730970034660824);
        vars.setValue(54, -8.875487287160695);
        vars.setValue(55, 0.0468739543956697);
        vars.setValue(56, -0.7852734225490905);
        vars.setValue(57, 0.017531522195257027);
        vars.setValue(58, 0.7681938978094821);
        vars.setValue(59, -0.22265049769722722);
        vars.setValue(60, -7.787103915157201);
        vars.setValue(61, 0.20258649058167544);
        vars.setValue(62, 3.89184653938446);
        vars.setValue(63, 0.4490870396661657);
        vars.setValue(64, 0.19458710272314095);
        vars.setValue(65, -0.23157678224607303);
        vars.setValue(66, -7.753146072367401);
        vars.setValue(67, -0.05129863223356834);
        vars.setValue(68, 3.89184653938771);
        vars.setValue(69, 0.4490870396647062);
        sim.modifyObjects();
        sim.saveInitialState();
        sim.setRandomSeed(1);
        const gravityLaw = new GravityLaw(10.0);
        sim.addForceLaw(gravityLaw);
        gravityLaw.addBodies(sim.getBodies());
        gravityLaw.setZeroEnergyLevel(zel);
    }
    ;
    static near_stable_connected_blocks_pile_test() {
        setTestName(groupName + 'near_stable_connected_blocks_pile_test');
        const sim = new ContactSim();
        const advance = new CollisionAdvance(sim);
        PileTest.near_stable_connected_blocks_pile_setup(sim, advance);
        const vars = makeVars(11 * 6);
        setBodyVars(sim, vars, 0, -5, 0, -5, 0, -0.7853982, 0);
        setBodyVars(sim, vars, 1, 5, 0, -5, 0, 0.7853982, 0);
        setBodyVars(sim, vars, 2, 10.5, 0, 7.5, 0, 0, 0);
        setBodyVars(sim, vars, 3, -10.5, 0, 7.5, 0, 0, 0);
        setBodyVars(sim, vars, 4, -1.3073175, 0, -7.6795634, -0, 3.9269908, 0);
        setBodyVars(sim, vars, 5, -0.6338649, 0, -8.3684045, -0, 3.9269908, -0);
        setBodyVars(sim, vars, 6, -0.8635095, 0, -7.4433597, -0, 16.4933614, 0);
        setBodyVars(sim, vars, 7, -0.2794896, 0, -8.8895748, 0, 21.2057504, -0);
        setBodyVars(sim, vars, 8, 0.1624361, 0.0000002, -8.8758426, 0.0000004, -0.7853982, -0.0000004);
        setBodyVars(sim, vars, 9, 0.7732544, -0, -7.8054609, 0, 3.9269908, -0);
        setBodyVars(sim, vars, 10, 0.1992748, -0, -7.7913801, 0, 3.9269908, -0);
        runTest(sim, advance, 6.0, vars, 0.001, NaN, 0, -1, -1);
        runTest(sim, advance, 10.0, null, NaN, 0.0, 0.001, 0);
        checkTightJoints(sim, 0.025);
    }
    ;
    static pile_config_1_setup(sim, advance) {
        sim.setSimRect(new DoubleRect(-7, 0, 7, 8));
        sim.addForceLaw(new DampingLaw(0.05, 0.15, sim.getSimList()));
        sim.setShowForces(false);
        sim.setCollisionHandling("serial grouped lastpass");
        sim.setCollisionAccuracy(0.6);
        sim.setRandomSeed(0);
        sim.setExtraAccel("velocity_and_distance");
        advance.setDiffEqSolver(new ModifiedEuler(sim));
        advance.setJointSmallImpacts(false);
        advance.setTimeStep(0.025);
        const buildRNG = new RandomLCG(594074265);
        const zel = PileConfig.makeVPit(sim, 9.348706704297266);
        const half = 3;
        const rest = 3;
        PileConfig.makeRandomBlocks(sim, rest, -9.9, 19, buildRNG);
        PileConfig.makeRandomBlocks(sim, half, -9, 16, buildRNG);
        PileConfig.makeConnectedBlocks(sim, 3, 21, 0);
        sim.setElasticity(0.8);
        const vars = sim.getVarsList();
        vars.setValue(28, -4.826036961610339);
        vars.setValue(29, 5.507262977202934);
        vars.setValue(30, 6.3686904268363635);
        vars.setValue(31, -7.953916554586297);
        vars.setValue(32, -2.4297073283893753);
        vars.setValue(33, -3.2910792690296575);
        vars.setValue(34, -0.39136366049574994);
        vars.setValue(35, 11.596594391854405);
        vars.setValue(36, 6.401989888500372);
        vars.setValue(37, -6.3050312680892455);
        vars.setValue(38, 2.833546728939748);
        vars.setValue(39, 11.84282384112968);
        vars.setValue(40, -2.6677254408578985);
        vars.setValue(41, 7.500061085685241);
        vars.setValue(42, 4.741186660019);
        vars.setValue(43, -7.976266185005914);
        vars.setValue(44, 8.581332736951099);
        vars.setValue(45, 12.121849497962163);
        vars.setValue(46, -0.5750827056429556);
        vars.setValue(47, 11.45349352174985);
        vars.setValue(48, 1.8528786215208863);
        vars.setValue(49, -2.623975579277876);
        vars.setValue(50, -4.0009341616207355);
        vars.setValue(51, -11.167073162011471);
        vars.setValue(52, 0.4901868773177362);
        vars.setValue(53, 10.209045158895574);
        vars.setValue(54, 0.8991047151294028);
        vars.setValue(55, 9.426218453147095);
        vars.setValue(56, -1.0250437996613715);
        vars.setValue(57, 58.10877825300471);
        vars.setValue(58, 0.38386358504151985);
        vars.setValue(59, -1.1422881143836392);
        vars.setValue(60, 3.346206103148756);
        vars.setValue(61, 6.623070984882051);
        vars.setValue(62, 23.57586164417926);
        vars.setValue(63, 35.37119888296379);
        vars.setValue(64, -0.674508123401993);
        vars.setValue(65, -7.960427953540814);
        vars.setValue(66, 4.248616367886464);
        vars.setValue(67, 0.683655285060861);
        vars.setValue(68, -2.0251585739061584);
        vars.setValue(69, -8.34224694613519);
        vars.setValue(70, -1.2106565077697318);
        vars.setValue(71, -9.494215020773796);
        vars.setValue(72, 4.065566370010308);
        vars.setValue(73, 5.146494032387025);
        vars.setValue(74, -2.0251585739061584);
        vars.setValue(75, -8.34224694613518);
        sim.modifyObjects();
        sim.saveInitialState();
        sim.setRandomSeed(0);
        const gravityLaw = new GravityLaw(10.0);
        sim.addForceLaw(gravityLaw);
        gravityLaw.addBodies(sim.getBodies());
        gravityLaw.setZeroEnergyLevel(zel);
    }
    ;
    static pile_config_1_test() {
        setTestName(groupName + 'pile_config_1_test');
        const sim = new ContactSim();
        const advance = new CollisionAdvance(sim);
        PileTest.pile_config_1_setup(sim, advance);
        runTest(sim, advance, 9.0, null, NaN, NaN, 0, -1, -1);
        runTest(sim, advance, 15.0, null, NaN, 0.0, 0.001, 0);
        checkTightJoints(sim, 0.025);
    }
    ;
    static pile_10_random_blocks_setup(sim, advance) {
        sim.setSimRect(new DoubleRect(-3.5, -9.2, 3.5, -1));
        sim.addForceLaw(new DampingLaw(0.05, 0.15, sim.getSimList()));
        sim.setShowForces(false);
        sim.setCollisionHandling("serial grouped lastpass");
        sim.setCollisionAccuracy(0.6);
        sim.setExtraAccel("velocity");
        advance.setDiffEqSolver(new ModifiedEuler(sim));
        advance.setJointSmallImpacts(true);
        advance.setTimeStep(0.025);
        const buildSeed = 0;
        const seed = 13;
        sim.setRandomSeed(seed);
        const zel = PileConfig.makeVPit(sim);
        const buildRNG = new RandomLCG(buildSeed);
        PileConfig.makeRandomBlocks(sim, 10, -5.5, -2, buildRNG);
        sim.setElasticity(0.5);
        const gravityLaw = new GravityLaw(3.0);
        sim.addForceLaw(gravityLaw);
        gravityLaw.addBodies(sim.getBodies());
        gravityLaw.setZeroEnergyLevel(zel);
    }
    ;
    static pile_10_random_blocks() {
        setTestName(groupName + 'pile_10_random_blocks');
        const sim = new ContactSim();
        const advance = new CollisionAdvance(sim);
        PileTest.pile_10_random_blocks_setup(sim, advance);
        const vars = makeVars(14 * 6);
        setBodyVars(sim, vars, 0, -5, 0, -5, 0, -0.7853982, 0);
        setBodyVars(sim, vars, 1, 5, 0, -5, 0, 0.7853982, 0);
        setBodyVars(sim, vars, 2, 10.5, 0, 7.5, 0, 0, 0);
        setBodyVars(sim, vars, 3, -10.5, 0, 7.5, 0, 0, 0);
        setBodyVars(sim, vars, 4, -1.4688341, 0, -7.366921, -0, 5.507218, -0);
        setBodyVars(sim, vars, 5, -1.2888741, 0, -6.8611893, -0, 2.3604327, 0);
        setBodyVars(sim, vars, 6, -1.0893343, 0, -7.5132601, -0, 0.7919455, -0);
        setBodyVars(sim, vars, 7, -0.5829513, 0, -8.0161773, -0, 0.7896626, 0);
        setBodyVars(sim, vars, 8, -0.0678191, -0, -8.6057224, -0, 0.7853285, -0);
        setBodyVars(sim, vars, 9, 0.1123147, -0, -7.4965551, -0, -0.771069, -0);
        setBodyVars(sim, vars, 10, 0.0843676, 0, -7.8743626, -0, 0.8066319, 0);
        setBodyVars(sim, vars, 11, 0.7205631, 0, -8.2682827, -0, 0.7924107, 0);
        setBodyVars(sim, vars, 12, 0.7795149, 0, -7.7446799, 0, 3.9304584, -0);
        setBodyVars(sim, vars, 13, 0.3485821, 0, -6.9817453, -0, 2.3673243, -0);
        const startTime = Util.systemTime();
        runTest(sim, advance, 8.0, vars, 0.01);
        runTest(sim, advance, 10.0, null, NaN, 0.0, 0.00001, 0);
        const s = ' collisions=' + advance.getCollisionTotals().getCollisions()
            + ', steps=' + advance.getCollisionTotals().getSteps()
            + ', contacts=' + sim.getNumContacts();
        myPrintln('info: ' + testName + s);
    }
    ;
    static pile_10_perf() {
        const testName = 'pile_10_perf';
        const expected = perfExpected(testName);
        const startTime = Util.systemTime();
        const repeats = 10;
        for (let i = 0; i < repeats; i++) {
            PileTest.pile_10_random_blocks();
        }
        const duration = (Util.systemTime() - startTime) / repeats;
        setTestName(groupName + testName);
        const s = perfResult(duration, expected);
        const timeLimit = getPerfLimit(expected);
        reportTestResults(duration < timeLimit, 'performance', s);
    }
    ;
    static pile_make_v_pit(sim, advance, damping, gravity) {
        sim.addForceLaw(new DampingLaw(damping, 0.15, sim.getSimList()));
        sim.setShowForces(false);
        sim.setCollisionHandling("serial grouped lastpass");
        sim.setCollisionAccuracy(0.6);
        sim.setExtraAccel("velocity");
        advance.setDiffEqSolver(new ModifiedEuler(sim));
        advance.setJointSmallImpacts(true);
        advance.setTimeStep(0.025);
        sim.setRandomSeed(14);
        const zel = PileConfig.makeVPit(sim);
        const gravityLaw = new GravityLaw(gravity, sim.getSimList());
        sim.addForceLaw(gravityLaw);
        gravityLaw.setZeroEnergyLevel(zel);
    }
    ;
    static pile_uniform_balls_setup(sim, advance) {
        PileTest.pile_make_v_pit(sim, advance, 0.05, 3.0);
        const rect = new DoubleRect(-9, 5, 9, 20);
        PileConfig.makeUniformBlocks(sim, rect, true, 2.0, 0.2, 10);
        sim.setElasticity(0.8);
    }
    ;
    static pile_uniform_blocks_setup(sim, advance) {
        PileTest.pile_make_v_pit(sim, advance, 0.05, 3.0);
        const rect = new DoubleRect(-9, 5, 9, 20);
        PileConfig.makeUniformBlocks(sim, rect, false, 2.0, 0.2, 10);
        sim.setElasticity(0.8);
    }
    ;
    static pile_20_random_blocks_setup(sim, advance) {
        sim.setSimRect(new DoubleRect(-3.5, -9.2, 3.5, -1));
        sim.addForceLaw(new DampingLaw(0.05, 0.15, sim.getSimList()));
        sim.setShowForces(false);
        sim.setCollisionHandling("serial grouped lastpass");
        sim.setCollisionAccuracy(0.6);
        sim.setExtraAccel("velocity");
        advance.setDiffEqSolver(new ModifiedEuler(sim));
        advance.setJointSmallImpacts(true);
        advance.setTimeStep(0.025);
        const buildSeed = 0;
        const seed = 14;
        sim.setRandomSeed(seed);
        const zel = PileConfig.makeVPit(sim);
        const buildRNG = new RandomLCG(buildSeed);
        PileConfig.makeRandomBlocks(sim, 10, -5.5, -2, buildRNG);
        PileConfig.makeRandomBlocks(sim, 10, -6, -0.5, buildRNG);
        sim.setElasticity(0.5);
        const gravityLaw = new GravityLaw(3.0);
        sim.addForceLaw(gravityLaw);
        gravityLaw.addBodies(sim.getBodies());
        gravityLaw.setZeroEnergyLevel(zel);
    }
    ;
    static pile_20_random_blocks() {
        setTestName(groupName + 'pile_20_random_blocks');
        const sim = new ContactSim();
        const advance = new CollisionAdvance(sim);
        PileTest.pile_20_random_blocks_setup(sim, advance);
        if (!Util.isChrome()) {
            throw '';
        }
        const startTime = Util.systemTime();
        runTest(sim, advance, 9.0, null, NaN);
        runTest(sim, advance, 12.0, null, NaN, 0.0, 0.00001, 0);
        const s = ' collisions=' + advance.getCollisionTotals().getCollisions()
            + ', steps=' + advance.getCollisionTotals().getSteps()
            + ', contacts=' + sim.getNumContacts();
        myPrintln('info: ' + testName + s);
    }
    ;
    static pile_20_perf() {
        const testName = 'pile_20_perf';
        const expected = perfExpected(testName);
        const startTime = Util.systemTime();
        PileTest.pile_20_random_blocks();
        const duration = Util.systemTime() - startTime;
        setTestName(groupName + testName);
        const s = perfResult(duration, expected);
        const timeLimit = getPerfLimit(expected);
        reportTestResults(duration < timeLimit, 'performance', s);
    }
    ;
    static additive_pile_setup_(sim, advance) {
        sim.addForceLaw(new DampingLaw(0.05, 0.15, sim.getSimList()));
        sim.setCollisionHandling("serial grouped lastpass");
        sim.setDistanceTol(0.01);
        sim.setCollisionAccuracy(0.6);
        sim.setRandomSeed(99999);
        sim.setExtraAccel("velocity");
        advance.setDiffEqSolver(new ModifiedEuler(sim));
        advance.setJointSmallImpacts(true);
        advance.setTimeStep(0.025);
        const zel = PileConfig.makeVPit(sim);
        sim.addForceLaw(new GravityLaw(3.0, sim.getSimList()));
    }
    ;
    static additive_pile_test(square, start_num_blocks) {
        const sim = new ContactSim();
        const advance = new CollisionAdvance(sim);
        PileTest.additive_pile_setup_(sim, advance);
        let simTime = 0;
        const TIME_SPAN = 5.0;
        const LIMIT_TIME = 0.9 * TIME_SPAN;
        let num_blocks = 0;
        const startTime = Util.systemTime();
        while (num_blocks < start_num_blocks) {
            simTime += 1.0;
            num_blocks++;
            PileTest.add_block_and_run(sim, advance, simTime, square);
        }
        while (true) {
            simTime += TIME_SPAN;
            num_blocks++;
            const realTime = PileTest.add_block_and_run(sim, advance, simTime, square);
            myPrintln('num_bodies=' + num_blocks
                + ' time=' + Util.NF2(realTime)
                + ' limit=' + Util.NF2(LIMIT_TIME)
                + ' totalTime=' + Util.NF5(Util.systemTime() - startTime)
                + ' contacts=' + sim.getNumContacts());
            if (realTime > LIMIT_TIME) {
                break;
            }
        }
        return num_blocks;
    }
    ;
    static add_block_and_run(sim, advance, runUntil, square) {
        const startTime = Util.systemTime();
        const p = square ? Shapes.makeBlock(1.0, 1.0, 'block')
            : Shapes.makeBall(0.5, 'ball');
        p.setPosition(new Vector(0.5, 0), 0.5);
        p.setElasticity(0.5);
        sim.addBody(p);
        runTest(sim, advance, runUntil);
        return (Util.systemTime() - startTime);
    }
    ;
    static additive_pile_square_test(expectedBlocks) {
        setTestName(groupName + 'additive_pile_square_test');
        const startTime = Util.systemTime();
        const num_blocks = PileTest.additive_pile_test(true, expectedBlocks - 3);
        const totalTime = Util.systemTime() - startTime;
        const s = 'reached ' + num_blocks
            + ' blocks; expected=' + expectedBlocks
            + ' totalTime=' + Util.NF5(totalTime);
        myPrintln('additive_pile_square_test ' + s);
        reportTestResults(num_blocks >= expectedBlocks, 'performance', s);
    }
    ;
    static additive_pile_circle_test(expectedBlocks) {
        setTestName(groupName + 'additive_pile_circle_test');
        const startTime = Util.systemTime();
        const num_blocks = PileTest.additive_pile_test(false, expectedBlocks - 3);
        const totalTime = Util.systemTime() - startTime;
        const s = 'reached ' + num_blocks
            + ' blocks; expected=' + expectedBlocks
            + ' totalTime=' + Util.NF5(totalTime);
        myPrintln('additive_pile_circle_test ' + s);
        reportTestResults(num_blocks >= expectedBlocks, 'performance', s);
    }
    ;
}
