import { Util } from "../lab/util/Util.js";
import { ExpectedPerf } from './ExpectedPerf.js';
export var testName = '';
export function setTestName(name) {
    testName = name;
}
;
const ABORT_ON_FAIL = false;
const BROWSER_CHROME = 'Chrome';
const BROWSER_FIREFOX = 'Firefox';
const BROWSER_SAFARI = 'Safari';
const testFns = new Array();
let testsFailed = 0;
let output = null;
function runFunction(testFunc) {
    try {
        testFunc();
    }
    catch (e) {
        testsFailed += 1;
        myPrintln('***FAILED*** ' + testName + ' ' + e, true);
    }
}
;
export function schedule(testFunc) {
    testFns.push(() => { runFunction(testFunc); });
}
;
export function runTests() {
    const testFunc = testFns.shift();
    if (testFunc !== undefined) {
        testFunc();
        setTimeout(runTests, 10);
    }
}
;
export function startTests() {
    testsFailed = 0;
    const test_results = document.getElementById('test_results');
    if (!Util.isObject(test_results)) {
        throw '<p> element with id="test_results" not found';
    }
    output = test_results;
    const d = new Date();
    myPrintln(d.toDateString() + ' ' + d.toTimeString());
    myPrintln('myPhysicsLab version = ' + Util.VERSION);
    myPrintln('compiled ' + Util.COMPILE_TIME);
    myPrintln('machine = ' + getMachineName());
    myPrintln('browser = ' + getBrowserName());
    const nav = navigator;
    if (nav != null) {
        myPrintln('userAgent = ' + nav.userAgent);
        myPrintln('platform = ' + nav.platform);
    }
    myPrintln('Util.DEBUG = ' + Util.DEBUG);
    if (Util.DEBUG) {
        let a = 1;
        try {
            Util.assert(false);
            a = 2;
        }
        catch (e) {
            myPrintln('asserts are working');
        }
        if (a == 2) {
            throw 'asserts are not working';
        }
    }
    else {
        myPrintln('NOTE: asserts are NOT enabled');
    }
}
;
export function finishTests() {
    const f = testsFailed;
    if (f > 0) {
        myPrintln('Tests finished -- ' + f + ' TESTS FAILED', true);
    }
    else {
        myPrintln('Tests finished and passed.');
    }
    const d = new Date();
    myPrintln(d.toDateString() + ' ' + d.toTimeString());
}
;
export function startTest(n) {
    testName = n;
    myPrintln(n);
}
;
export function myPrintln(s, opt_error, opt_warning) {
    console.log(s);
    if (output != null) {
        if (opt_error) {
            s = '<span class="error">' + s + '</span>';
        }
        if (opt_warning) {
            s = '<span class="warning">' + s + '</span>';
        }
        output.innerHTML += s + '<br>';
        const docElement = document.documentElement;
        const documentHeight = docElement.offsetHeight;
        const viewportHeight = window.innerHeight;
        window.scrollTo(0, documentHeight - viewportHeight);
    }
}
;
export function reportTestResults(passed, testType, reason) {
    let s = testName + ' [' + testType + ']';
    if (passed) {
        if (testType == 'performance') {
            s += ' ' + reason;
        }
        const warning = testName.match(/.*error.*/) != null;
        myPrintln('passed: ' + s, false, warning);
    }
    else {
        testsFailed += 1;
        myPrintln('FAILED ' + s, true);
        if (typeof reason === 'string' && reason.length > 0) {
            myPrintln(reason, true);
        }
        console.trace();
        if (ABORT_ON_FAIL) {
            throw (new Error());
        }
    }
}
;
export function getMachineName() {
    if (window.MYPHYSICSLAB_MACHINE_NAME !== undefined) {
        const s = window.MYPHYSICSLAB_MACHINE_NAME;
        if (typeof s === 'string') {
            return s;
        }
    }
    if (Util.DEBUG) {
        console.log('MYPHYSICSLAB_MACHINE_NAME not defined');
    }
    return 'UNKNOWN_MACHINE';
}
;
export function getBrowserName() {
    const nav = navigator;
    if (nav == null)
        return 'unknown';
    if (nav.userAgent.match(/.*Chrome.*/) != null)
        return BROWSER_CHROME;
    else if (nav.userAgent.match(/.*Firefox.*/) != null)
        return BROWSER_FIREFOX;
    else if (nav.userAgent.match(/.*Safari.*/) != null)
        return BROWSER_SAFARI;
    else
        return 'other';
}
;
export function assertElementsEquals(expected, arr) {
    if (!Array.isArray(arr)) {
        reportTestResults(false, 'assert', 'not an array ' + arr);
        return;
    }
    if (expected.length != arr.length) {
        reportTestResults(false, 'assert', 'expected array length '
            + expected.length + ' but found ' + arr.length);
        return;
    }
    for (let i = 0, n = expected.length; i < n; i++) {
        if (expected[i] != arr[i]) {
            const s = 'expected=' + expected[i] + ' but was actual=' + arr[i];
            reportTestResults(false, 'assert', 'array elements not equal ' + s);
        }
    }
}
;
export function assertEquals(expected, value) {
    if (value !== expected) {
        const s = 'expected=' + expected + ' but was actual=' + value;
        reportTestResults(false, 'value', s);
    }
}
;
export function assertFalse(value) {
    if (value) {
        reportTestResults(false, 'assert');
    }
}
;
export function assertLessThan(value, limit) {
    if (value > limit) {
        const s = 'value of ' + value + ' exceeds limit of ' + limit;
        reportTestResults(false, 'value', s);
    }
}
;
export function assertNaN(value) {
    if (!isNaN(value)) {
        reportTestResults(false, 'assert', 'value is not NaN ' + value);
    }
}
;
export function assertNotNull(value) {
    if (value === null) {
        reportTestResults(false, 'assert', 'value is null');
    }
}
;
export function assertNull(value) {
    if (value !== null) {
        reportTestResults(false, 'assert', 'value is not null ' + value);
    }
}
;
export function assertNotThrows(func) {
    try {
        func();
    }
    catch (e) {
        reportTestResults(false, 'exception', 'exception should not occur');
    }
}
;
export function assertRoughlyEquals(expected, value, tolerance) {
    if (Math.abs(expected - value) > tolerance) {
        const s = 'expected=' + expected + ' but was actual=' + value
            + ' tolerance=' + tolerance;
        reportTestResults(false, 'value', s);
    }
}
;
export function assertThrows(func) {
    try {
        func();
        reportTestResults(false, 'assert', 'expected exception did not occur');
        return '';
    }
    catch (e) {
        return String(e);
    }
}
;
export function assertTrue(value) {
    if (!value) {
        reportTestResults(false, 'assert');
    }
}
;
export function assertUndefined(value) {
    if (value !== undefined) {
        reportTestResults(false, 'assert');
    }
}
;
export function getPerfLimit(expected) {
    return 1.20 * expected;
}
;
export function perfResult(duration, expected) {
    return 'time=' + Util.NF2(duration) + ' expected=' + Util.NF2(expected)
        + '  (' + (Util.NF1S(100 * duration / expected - 100)) + '%)';
}
;
export function perfExpected(testName, defaultTime = 10000) {
    const machine = getMachineName();
    const browser = getBrowserName();
    const compiled = 'simple';
    const e0 = ExpectedPerf;
    Util.assert(Util.isObject(e0));
    let err = 'no expected results for machine: ' + machine;
    const e1 = e0[machine];
    if (!Util.isObject(e1)) {
        myPrintln(err, true);
        return defaultTime;
    }
    err += ', browser: ' + browser;
    const e2 = e1[browser];
    if (!Util.isObject(e2)) {
        myPrintln(err, true);
        return defaultTime;
    }
    err += ', test: ' + testName;
    const e4 = e2[testName];
    if (typeof e4 !== 'number') {
        myPrintln(err, true);
        return defaultTime;
    }
    else {
        return e4;
    }
}
;
