import { PileTest } from './PileTest.js';
import { CircleCircleTest } from './CircleCircleTest.js';
import { CircleStraightTest } from './CircleStraightTest.js';
import { DoNothingTest } from './DoNothingTest.js';
import { JointTest } from './JointTest.js';
import { MiscellanyTest } from './MiscellanyTest.js';
import { MultipleCollisionTest } from './MultipleCollisionTest.js';
import { RopeTest } from './RopeTest.js';
import { SpeedTest } from './SpeedTest.js';
import { StraightStraightTest } from './StraightStraightTest.js';
import { AbstractSubject } from '../lab/util/AbstractSubject.js';
import { CheckBoxControl } from '../lab/controls/CheckBoxControl.js';
import { ChoiceControl } from '../lab/controls/ChoiceControl.js';
import { Clock } from '../lab/util/Clock.js';
import { CollisionAdvance } from '../lab/model/CollisionAdvance.js';
import { CommonControls } from '../sims/common/CommonControls.js';
import { ContactSim } from '../lab/engine2D/ContactSim.js';
import { DampingLaw } from '../lab/model/DampingLaw.js';
import { DiffEqSolverSubject } from '../lab/model/DiffEqSolverSubject.js';
import { DisplayClock } from '../lab/view/DisplayClock.js';
import { DisplayShape } from '../lab/view/DisplayShape.js';
import { DoubleRect } from '../lab/util/DoubleRect.js';
import { ElasticitySetter } from '../sims/engine2D/ElasticitySetter.js';
import { EnergyBarGraph } from '../lab/graph/EnergyBarGraph.js';
import { GenericObserver, ParameterBoolean, ParameterNumber } from '../lab/util/Observe.js';
import { Gravity2Law } from '../lab/model/Gravity2Law.js';
import { GravityLaw } from '../lab/model/GravityLaw.js';
import { GroupControl } from '../lab/controls/GroupControl.js';
import { LabelControl } from '../lab/controls/LabelControl.js';
import { NumericControl } from '../lab/controls/NumericControl.js';
import { PathObserver } from '../sims/roller/PathObserver.js';
import { PileConfig } from '../sims/engine2D/PileConfig.js';
import { Polygon } from '../lab/engine2D/Polygon.js';
import { RigidBodyEventHandler } from '../lab/app/RigidBodyEventHandler.js';
import { RigidBodyObserver } from '../sims/engine2D/RigidBodyObserver.js';
import { RigidBodySim } from '../lab/engine2D/RigidBodySim.js';
import { SimController } from '../lab/app/SimController.js';
import { SimRunner } from '../lab/app/SimRunner.js';
import { SimView } from '../lab/view/SimView.js';
import { StandardGraph1 } from '../sims/common/StandardGraph1.js';
import { Util } from '../lab/util/Util.js';
import { Vector } from '../lab/util/Vector.js';
import { VerticalLayout } from '../sims/common/VerticalLayout.js';
import { VarsHistory } from '../lab/graph/VarsHistory.js';
import { FunctionVariable } from '../lab/model/FunctionVariable.js';
import { ClockTask } from '../lab/util/Clock.js';
import { GenericMemo } from '../lab/util/Memo.js';
import { DisplayText } from '../lab/view/DisplayText.js';
export class TestViewerApp extends AbstractSubject {
    constructor(elem_ids) {
        Util.setErrorHandler();
        super('TEST_VIEWER_APP');
        this.groups_ = [];
        this.groupNames_ = [];
        this.groupSelected_ = -1;
        this.tests_ = [];
        this.testNames_ = [];
        this.testSelected_ = -1;
        this.startOnLoad_ = true;
        this.layout = new VerticalLayout(elem_ids);
        this.layout.getSimCanvas().setBackground('black');
        const sim_controls = this.layout.getSimControls();
        this.terminal = this.layout.getTerminal();
        const simCanvas = this.layout.getSimCanvas();
        const graphCanvas = this.layout.getGraphCanvas();
        const div_sim = this.layout.getSimDiv();
        this.sim = new ContactSim();
        this.terminal.setAfterEval(() => this.sim.modifyObjects());
        new GenericObserver(this.sim, _evt => this.sim.modifyObjects(), 'modifyObjects after parameter or variable change');
        this.sim.setShowForces(true);
        this.sim.setCollisionHandling("serial grouped lastpass");
        this.sim.setExtraAccel("velocity_and_distance");
        this.simList = this.sim.getSimList();
        this.varsList = this.sim.getVarsList();
        this.varsList.setHistory(true);
        this.advance = new CollisionAdvance(this.sim);
        this.advance.setJointSmallImpacts(true);
        this.advance.setDebugLevel(3);
        this.simRect = new DoubleRect(-6, -6, 6, 6);
        this.simView = new SimView('simView', this.simRect);
        this.displayList = this.simView.getDisplayList();
        simCanvas.addView(this.simView);
        this.statusView = new SimView('status', new DoubleRect(-10, -10, 10, 10));
        simCanvas.addView(this.statusView);
        this.axes = CommonControls.makeAxes(this.simView, true);
        this.simRun = new SimRunner(this.advance);
        this.simRun.setTimeStep(0.025);
        this.simRun.addCanvas(simCanvas);
        this.simRun.getParameterNumber(SimRunner.en.TIME_STEP).setSignifDigits(7);
        this.clock = this.simRun.getClock();
        this.clock.getParameterNumber(Clock.en.TIME_RATE).setSignifDigits(5);
        this.rbeh = new RigidBodyEventHandler(this.sim, this.clock);
        this.simCtrl = new SimController(simCanvas, this.rbeh);
        this.rbo = new RigidBodyObserver(this.simList, this.simView.getDisplayList());
        this.pathObserver = new PathObserver(this.simList, this.simView, null);
        this.addGroup_(StraightStraightTest, 'Straight/Straight');
        this.addGroup_(CircleStraightTest, 'Circle/Straight');
        this.addGroup_(CircleCircleTest, 'Circle/Circle');
        this.addGroup_(DoNothingTest, 'DoNothingGrinder');
        this.addGroup_(JointTest, 'Joints');
        this.addGroup_(MiscellanyTest, 'Miscellany');
        this.addGroup_(MultipleCollisionTest, 'Multiple Collision');
        this.addGroup_(PileTest, 'Pile');
        this.addGroup_(RopeTest, 'Ropes');
        this.addGroup_(SpeedTest, 'Speed');
        let pn;
        let ps;
        let pb;
        const pbc = CommonControls.makePlaybackControls(this.simRun);
        this.prependControl(pbc);
        let br = new GroupControl('BR', document.createElement('BR'), []);
        this.prependControl(br);
        this.addParameter(pn = new ParameterNumber(this, TestViewerApp.en.GROUP, TestViewerApp.i18n.GROUP, () => this.getGroup(), a => this.setGroup(a), this.groupNames_, Util.range(this.groupNames_.length)));
        this.prependControl(new ChoiceControl(pn, ''));
        this.testParam_ = new ParameterNumber(this, TestViewerApp.en.TEST, TestViewerApp.i18n.TEST, () => this.getTest(), a => this.setTest(a), this.testNames_, Util.range(this.testNames_.length));
        this.addParameter(this.testParam_);
        this.prependControl(new ChoiceControl(this.testParam_, ''));
        this.addParameter(pb = new ParameterBoolean(this, TestViewerApp.en.START_ON_LOAD, TestViewerApp.i18n.START_ON_LOAD, () => this.getStartOnLoad(), a => this.setStartOnLoad(a)));
        this.prependControl(new CheckBoxControl(pb));
        br = new GroupControl('BR', document.createElement('BR'), []);
        this.prependControl(br);
        this.elasticity = new ElasticitySetter(this.sim);
        pn = this.elasticity.getParameterNumber(ElasticitySetter.en.ELASTICITY);
        this.addControl(new NumericControl(pn));
        if (0 == 1 && Util.DEBUG) {
            this.addControl(new LabelControl('compiled ' + Util.COMPILE_TIME));
        }
        br = new GroupControl('BR', document.createElement('BR'), []);
        this.addControl(br);
        pb = this.sim.getParameterBoolean(RigidBodySim.en.SHOW_FORCES);
        this.addControl(new CheckBoxControl(pb));
        this.energyGraph = new EnergyBarGraph(this.sim);
        this.showEnergyParam = CommonControls.makeShowEnergyParam(this.energyGraph, this.statusView, this);
        this.addControl(new CheckBoxControl(this.showEnergyParam));
        this.displayClock = new DisplayClock(() => this.sim.getTime(), () => this.clock.getRealTime(), 2, 2);
        this.displayClock.setPosition(new Vector(8, 4));
        this.showClockParam = CommonControls.makeShowClockParam(this.displayClock, this.statusView, this);
        this.addControl(new CheckBoxControl(this.showClockParam));
        const panzoom = CommonControls.makePanZoomControls(this.simView, true, () => this.simView.setSimRect(this.simRect));
        this.layout.getSimDiv().appendChild(panzoom);
        this.panZoomParam = CommonControls.makeShowPanZoomParam(panzoom, this);
        this.panZoomParam.setValue(false);
        this.addControl(new CheckBoxControl(this.panZoomParam));
        const bm = CommonControls.makeBackgroundMenu(this.layout.getSimCanvas());
        this.addControl(bm);
        br = new GroupControl('BR', document.createElement('BR'), []);
        this.addControl(br);
        pn = this.simRun.getParameterNumber(SimRunner.en.TIME_STEP);
        this.addControl(new NumericControl(pn));
        pn = this.simRun.getClock().getParameterNumber(Clock.en.TIME_RATE);
        this.addControl(new NumericControl(pn));
        this.diffEqSolver = new DiffEqSolverSubject(this.sim, this.sim, this.advance);
        ps = this.diffEqSolver.getParameterString(DiffEqSolverSubject.en.DIFF_EQ_SOLVER);
        this.addControl(new ChoiceControl(ps));
        this.graph = new StandardGraph1(this.varsList, this.layout.getGraphCanvas(), this.layout.getGraphControls(), this.layout.getGraphDiv(), this.simRun, 'inline');
        const p = this.rbo.protoPolygon;
        p.setFillStyle('rgba(51,204,255,0.5)');
        p.setNameColor('gray');
        p.setNameFont('12pt sans-serif');
        p.setDrawCenterOfMass(true);
        p.setDrawDragPoints(true);
        let subjects = [
            this,
            this.sim,
            this.diffEqSolver,
            this.simRun,
            this.clock,
            this.simView,
            this.statusView,
            this.layout.getSimCanvas(),
            this.layout.getGraphCanvas(),
            this.elasticity
        ];
        subjects = subjects.concat(this.graph.getSubjects());
        this.easyScript = CommonControls.makeEasyScript(subjects, [this.varsList], this.simRun, this.terminal);
        this.addControl(CommonControls.makeURLScriptButton(this.easyScript, this.simRun));
    }
    ;
    toString() {
        return this.toStringShort().slice(0, -1)
            + ', simList: ' + this.simList.toStringShort()
            + ', simCtrl: ' + this.simCtrl.toStringShort()
            + ', advance: ' + this.advance
            + ', simRect: ' + this.simRect
            + ', simView: ' + this.simView.toStringShort()
            + ', statusView: ' + this.statusView.toStringShort()
            + ', axes: ' + this.axes.toStringShort()
            + ', simRun: ' + this.simRun.toStringShort()
            + ', clock: ' + this.clock.toStringShort()
            + ', energyGraph: ' + this.energyGraph.toStringShort()
            + ', displayClock: ' + this.displayClock.toStringShort()
            + ', graph: ' + this.graph.toStringShort()
            + ', layout: ' + this.layout.toStringShort()
            + ', easyScript: ' + this.easyScript.toStringShort()
            + ', terminal: ' + this.terminal
            + super.toString();
    }
    ;
    getClassName() {
        return 'TestViewerApp';
    }
    ;
    defineNames(myName) {
        this.terminal.addRegex('advance|axes|clock|diffEqSolver|displayClock'
            + '|energyGraph|graph|layout|sim|simCtrl|simList|simRect|simRun'
            + '|simView|statusView|terminal|varsList|displayList', myName + '.');
        this.terminal.addRegex('simCanvas', myName + '.layout.');
        this.terminal.addRegex('gravityLaw|dampingLaw|easyScript|elasticity', myName + '.');
    }
    ;
    getGroup() {
        return this.groupSelected_;
    }
    ;
    setGroup(groupIndex) {
        if (groupIndex < 0 || groupIndex >= this.groupNames_.length) {
            throw 'setGroup: groupIndex=' + groupIndex
                + ' range=' + this.groupNames_.length;
        }
        if (this.groupSelected_ != groupIndex) {
            this.groupSelected_ = groupIndex;
            this.testSelected_ = -1;
            this.tests_.length = 0;
            this.testNames_.length = 0;
            this.addTestsFrom_(this.groups_[groupIndex]);
            this.testParam_.setChoices(this.testNames_, Util.range(this.testNames_.length));
            this.testParam_.setValue(0);
            this.broadcastParameter(TestViewerApp.en.GROUP);
        }
    }
    ;
    getTest() {
        return this.testSelected_;
    }
    ;
    setTest(index) {
        if (index < 0 || index >= this.testNames_.length) {
            throw 'setTest: index=' + index + ' range=' + this.testNames_.length;
        }
        if (this.testSelected_ != index) {
            this.testSelected_ = index;
            this.startTest_(index);
            this.broadcastParameter(TestViewerApp.en.TEST);
        }
    }
    ;
    addGroup_(c, name) {
        this.groups_.push(c);
        this.groupNames_.push(name);
    }
    ;
    getStartOnLoad() {
        return this.startOnLoad_;
    }
    ;
    setStartOnLoad(value) {
        if (this.startOnLoad_ != value) {
            this.startOnLoad_ = value;
            this.broadcastParameter(TestViewerApp.en.START_ON_LOAD);
        }
    }
    ;
    addTestsFrom_(c) {
        const nms = Object.getOwnPropertyNames(c);
        for (let i = 0; i < nms.length; i++) {
            let p = nms[i];
            if (typeof c[p] !== 'function')
                continue;
            if (!p.match(/.*_setup$/))
                continue;
            const nm = p.replace(/_setup$/, '').replace(/_/g, ' ');
            this.testNames_.push(nm);
            this.tests_.push(c[p]);
        }
    }
    ;
    startTest_(testIndex) {
        Util.assert(typeof testIndex === 'number');
        this.sim.setDistanceTol(0.01);
        this.sim.setVelocityTol(0.5);
        this.sim.setCollisionAccuracy(0.6);
        this.sim.cleanSlate();
        this.advance.reset();
        const groupName = this.groupNames_[this.groupSelected_];
        const testName = this.testNames_[testIndex];
        if (Util.DEBUG) {
            console.log('TestViewerApp.startTest group=' + groupName + ' test="' + testName + '"');
        }
        const showForce = this.sim.getShowForces();
        Polygon.ID = 1;
        this.tests_[testIndex](this.sim, this.advance);
        this.displayList.toArray().forEach(ds => {
            if (ds instanceof DisplayShape) {
                const p = ds.getMassObjects()[0];
                if (isFinite(p.getMass())) {
                    ds.setFillStyle(PileConfig.getRandomColor());
                }
                if (groupName == 'Miscellany' && testName == 'clock with gears') {
                    ds.setStrokeStyle('black');
                    if (p.nameEquals('escape wheel')) {
                        ds.setZIndex(-1);
                    }
                }
            }
        });
        this.simRun.setTimeStep(this.advance.getTimeStep());
        this.diffEqSolver.broadcastParameter(DiffEqSolverSubject.en.DIFF_EQ_SOLVER);
        this.sim.setShowForces(showForce);
        this.makeGravityControl_();
        this.makeDampingControl_();
        this.sim.saveInitialState();
        this.setSimRect_();
        this.easyScript.update();
        this.clock.setTime(this.sim.getTime());
        this.clock.setRealTime(this.sim.getTime());
        if (this.startOnLoad_) {
            this.clock.resume();
        }
        else {
            this.clock.pause();
        }
    }
    ;
    makeGravityControl_() {
        const g = this.sim.getForceLaws().find(f => f instanceof GravityLaw || f instanceof Gravity2Law);
        const e = Util.getElementById('gravity_control');
        if (g === undefined) {
            e.value = '';
            e.disabled = true;
            return;
        }
        else {
            this.gravityLaw = g;
            const p = this.gravityLaw.getParameterNumber(GravityLaw.en.GRAVITY);
            const nc = new NumericControl(p, e);
            nc.setEnabled(true);
        }
    }
    ;
    makeDampingControl_() {
        const g = this.sim.getForceLaws().find(f => f instanceof DampingLaw);
        const e = Util.getElementById('damping_control');
        if (g == null) {
            e.value = '';
            e.disabled = true;
            return;
        }
        else {
            this.dampingLaw = g;
            const p = this.dampingLaw.getParameterNumber(DampingLaw.en.DAMPING);
            const nc = new NumericControl(p, e);
            nc.setEnabled(true);
        }
    }
    ;
    setSimRect_() {
        const simRect = this.sim.getSimRect();
        if (simRect != null) {
            this.simRect = simRect;
        }
        else {
            const bods = this.sim.getSimList().toArray();
            if (bods.length == 0) {
                return;
            }
            let rect = bods[0].getBoundsWorld();
            let walls = false;
            bods.forEach(b => {
                rect = rect.union(b.getBoundsWorld());
                walls = walls || /^WALL*/.test(b.getName());
            });
            if (!walls) {
                rect = rect.scale(2.0);
            }
            this.simRect = rect;
        }
        this.simView.setSimRect(this.simRect);
    }
    ;
    setup() {
        this.setGroup(0);
        this.layout.showTerminal(true);
        this.layout.showControls(true);
        this.statusView.getDisplayList().add(this.displayClock);
        this.statusView.getDisplayList().add(this.energyGraph);
        this.terminal.parseURL();
        this.sim.saveInitialState();
        this.sim.modifyObjects();
    }
    ;
    addControl(control) {
        return this.layout.addControl(control);
    }
    ;
    prependControl(control) {
        const element = control.getElement();
        element.style.display = 'inline-block';
        this.layout.addControl(control, false);
        const e = Util.getElementById('damping_control');
        this.layout.getSimControls().insertBefore(element, e.parentNode);
        return control;
    }
    ;
    eval(script, output = true) {
        return this.terminal.eval(script, output);
    }
    ;
    static loadClass() {
        var f = FunctionVariable.toString;
        f = VarsHistory.toString;
        f = GenericMemo.toString;
        f = DisplayText.toString;
        f = ClockTask.toString;
    }
    ;
    start() {
        this.simRun.startFiring();
    }
    ;
}
TestViewerApp.en = {
    GROUP: 'group',
    TEST: 'test',
    START_ON_LOAD: 'start on load'
};
TestViewerApp.de_strings = {
    GROUP: 'Gruppe',
    TEST: 'Prüfung',
    START_ON_LOAD: 'beginn nach laden'
};
TestViewerApp.i18n = Util.LOCALE === 'de' ? TestViewerApp.de_strings : TestViewerApp.en;
Util.defineGlobal('test$TestViewerApp', TestViewerApp);
